# -*- coding: utf-8 -*-
# Copyright 2020 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)


from __future__ import absolute_import, division, print_function

__metaclass__ = type


import unittest
from ansible_collections.ansible.utils.plugins.module_utils.common.index_of import (
    index_of,
)
from ansible.template import Templar


class TestIndexOfFilter(unittest.TestCase):
    def setUp(self):
        self._tests = Templar(loader=None).environment.tests

    def test_fail_no_qualfier(self):
        obj, test, value = [1, 2], "@@", 1
        with self.assertRaises(Exception) as exc:
            index_of(obj, test, value, tests=self._tests)
        self.assertIn("the test '@@' was not found", str(exc.exception))
        obj, test, value, key = [{"a": 1}], "@@", 1, "a"
        with self.assertRaises(Exception) as exc:
            index_of(obj, test, value, key, tests=self._tests)
        self.assertIn("the test '@@' was not found", str(exc.exception))

    def test_fail_mixed_list(self):
        obj, test, value, key = [{"a": "b"}, True, 1, "a"], "==", "b", "a"
        with self.assertRaises(Exception) as exc:
            index_of(obj, test, value, key, tests=self._tests)
        self.assertIn("required to be dictionaries", str(exc.exception))

    def test_fail_key_not_valid(self):
        obj, test, value, key = [{"a": "b"}], "==", "b", [1, 2]
        with self.assertRaises(Exception) as exc:
            index_of(obj, test, value, key, tests=self._tests)
        self.assertIn("Unknown key type", str(exc.exception))

    def test_fail_on_missing(self):
        obj, test, value, key = [{"a": True}, {"c": False}], "==", True, "a"
        with self.assertRaises(Exception) as exc:
            index_of(
                obj, test, value, key, fail_on_missing=True, tests=self._tests
            )
        self.assertIn("'a' was not found", str(exc.exception))

    def test_just_test(self):
        """Limit to jinja < 2.11 tests"""
        objs = [
            # ([True], "true", 0),
            # ([False], "not false", []),
            # ([False, 5], "boolean", 0),
            # ([0, False], "false", 1),
            ([3, 4], "not even", 0),
            ([3, 4], "even", 1),
            ([3, 3], "even", []),
            ([3, 3, 3, 4], "odd", [0, 1, 2]),
            # ([3.3, 3.4], "float", [0, 1]),
        ]
        for entry in objs:
            obj, test, answer = entry
            result = index_of(obj, test, tests=self._tests)
            expected = answer
            self.assertEqual(result, expected)

    def test_simple_lists(self):
        objs = [
            ([1, 2, 3], "==", 2, 1),
            (["a", "b", "c"], "eq", "c", 2),
            ([True, False, 0, 1], "equalto", False, [1, 2]),
            ([True, False, "0", "1"], "==", False, 1),
            ([True, False, "", "1"], "==", False, 1),
            ([True, False, "", "1"], "in", False, 1),
            ([True, False, "", "1", "a"], "in", [False, "1"], [1, 3]),
            ([1, 2, 3, "a", "b", "c"], "!=", "c", [0, 1, 2, 3, 4]),
            ([1, 2, 3], "!<", 3, 2),
        ]
        for entry in objs:
            obj, test, value, answer = entry
            result = index_of(obj, test, value, tests=self._tests)
            expected = answer
            self.assertEqual(result, expected)

    def test_simple_dict(self):
        objs = [
            ([{"a": 1}], "==", 1, "a", 0),
            ([{"a": 1}], "==", 1, "b", []),
            ([{"a": 1}], "==", 2, "a", []),
            (
                [{"a": 1}, {"a": 1}, {"a": 1}, {"a": 2}],
                "==",
                1,
                "a",
                [0, 1, 2],
            ),
            (
                [{"a": "abc"}, {"a": "def"}, {"a": "ghi"}, {"a": "jkl"}],
                "ansible.builtin.match",
                "^a",
                "a",
                0,
            ),
            (
                [{"a": "abc"}, {"a": "def"}, {"a": "ghi"}, {"a": "jkl"}],
                "ansible.builtin.search",
                "e",
                "a",
                1,
            ),
        ]
        for entry in objs:
            obj, test, value, key, answer = entry
            result = index_of(obj, test, value, key, tests=self._tests)
            self.assertEqual(result, answer)



# -*- coding: utf-8 -*-
# Copyright 2020 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

import unittest

from ansible.errors import AnsibleError
from ansible_collections.ansible.utils.plugins.test.validate import validate

DATA = {
    "GigabitEthernet0/0/0/0": {
        "auto_negotiate": False,
        "counters": {
            "in_crc_errors": 0,
            "in_errors": 0,
            "rate": {"in_rate": 0, "out_rate": 0},
        },
        "description": "configured using Ansible",
        "duplex_mode": "full",
        "enabled": True,
        "line_protocol": "up",
        "mtu": 1514,
        "oper_status": "down",
        "type": "GigabitEthernet",
    },
    "GigabitEthernet0/0/0/1": {
        "auto_negotiate": False,
        "counters": {
            "in_crc_errors": 10,
            "in_errors": 0,
            "rate": {"in_rate": 0, "out_rate": 0},
        },
        "description": "# interface is configures with Ansible",
        "duplex_mode": "full",
        "enabled": False,
        "line_protocol": "up",
        "mtu": 1514,
        "oper_status": "up",
        "type": "GigabitEthernet",
    },
}

CRITERIA_CRC_ERROR_CHECK = {
    "type": "object",
    "patternProperties": {
        "^.*": {
            "type": "object",
            "properties": {
                "counters": {
                    "properties": {
                        "in_crc_errors": {"type": "number", "maximum": 0}
                    }
                }
            },
        }
    },
}

CRITERIA_ENABLED_CHECK = {
    "type": "object",
    "patternProperties": {
        "^.*": {"type": "object", "properties": {"enabled": {"enum": [True]}}}
    },
}

CRITERIA_OPER_STATUS_UP_CHECK = {
    "type": "object",
    "patternProperties": {
        "^.*": {
            "type": "object",
            "properties": {"oper_status": {"type": "string", "pattern": "up"}},
        }
    },
}

CRITERIA_IN_RATE_CHECK = {
    "type": "object",
    "patternProperties": {
        "^.*": {
            "type": "object",
            "properties": {
                "counters": {
                    "properties": {
                        "rate": {
                            "properties": {
                                "in_rate": {"type": "number", "maximum": 0}
                            }
                        }
                    }
                }
            },
        }
    },
}


class TestValidate(unittest.TestCase):
    def setUp(self):
        pass

    def test_invalid_argspec(self):
        """Check passing invalid argspec"""

        # missing required arguments
        args = [DATA]
        kwargs = {}
        with self.assertRaises(AnsibleError) as error:
            validate(*args, **kwargs)
        msg = "missing required arguments: criteria"
        self.assertTrue(
            set(str(error.exception).split()).issuperset(set(msg.split()))
        )

        kwargs = {
            "criteria": CRITERIA_IN_RATE_CHECK,
            "engine": "ansible.utils.sample",
        }
        with self.assertRaises(AnsibleError) as error:
            validate(*args, **kwargs)
        self.assertIn(
            "For engine 'ansible.utils.sample' error loading",
            str(error.exception),
        )

        args = ["invalid data"]
        kwargs = {
            "criteria": [CRITERIA_IN_RATE_CHECK],
            "engine": "ansible.utils.jsonschema",
        }
        with self.assertRaises(AnsibleError) as error:
            validate(*args, **kwargs)
        self.assertIn("'data' option value is invalid", str(error.exception))

        args = [DATA]
        kwargs = {
            "criteria": "invalid criteria",
            "engine": "ansible.utils.jsonschema",
        }
        with self.assertRaises(AnsibleError) as error:
            validate(*args, **kwargs)
        self.assertIn(
            "'criteria' option value is invalid", str(error.exception)
        )

    def test_invalid_validate_plugin_config_options(self):
        """Check passing invalid validate plugin options"""
        args = [DATA]
        kwargs = {
            "criteria": "invalid criteria",
            "engine": "ansible.utils.jsonschema",
            "draft": "draft0",
        }

        with self.assertRaises(AnsibleError) as error:
            validate(*args, **kwargs)
        self.assertIn(
            "value of draft must be one of: draft3, draft4, draft6, draft7, got: draft0",
            str(error.exception),
        )

    def test_invalid_data(self):
        """Check passing invalid data as per criteria"""
        args = [DATA]
        kwargs = {
            "criteria": [
                CRITERIA_ENABLED_CHECK,
                CRITERIA_OPER_STATUS_UP_CHECK,
                CRITERIA_CRC_ERROR_CHECK,
            ],
            "engine": "ansible.utils.jsonschema",
        }
        result = validate(*args, **kwargs)
        self.assertEqual(result, False)

    def test_valid_data(self):
        """Check passing valid data as per criteria"""
        args = [DATA]
        kwargs = {
            "criteria": CRITERIA_IN_RATE_CHECK,
            "engine": "ansible.utils.jsonschema",
        }
        result = validate(*args, **kwargs)
        self.assertEqual(result, True)



# -*- coding: utf-8 -*-
# Copyright 2021 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

"""
Unit test file for netaddr test plugin: mac
"""

from __future__ import absolute_import, division, print_function

__metaclass__ = type

import unittest
from ansible_collections.ansible.utils.plugins.test.mac import _mac


class TestMac(unittest.TestCase):
    def setUp(self):
        pass

    def test_invalid_data(self):
        """Check passing invalid argspec"""

        # missing argument
        with self.assertRaises(TypeError) as error:
            _mac()
        self.assertIn("argument", str(error.exception))

    def test_valid_data(self):
        """Check passing valid data as per criteria"""

        result = _mac(mac="02:16:3e:e4:16:f3")
        self.assertEqual(result, True)

        result = _mac(mac="02-16-3e-e4-16-f3")
        self.assertEqual(result, True)

        result = _mac(mac="0216.3ee4.16f3")
        self.assertEqual(result, True)

        result = _mac(mac="02163ee416f3")
        self.assertEqual(result, True)

        result = _mac(mac="string")
        self.assertEqual(result, False)



# -*- coding: utf-8 -*-
# Copyright 2020 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

import unittest
from ansible.errors import AnsibleError
from ansible.errors import AnsibleFilterError
from ansible_collections.ansible.utils.plugins.filter.from_xml import _from_xml

INVALID_DATA = '<netconf-state xmlns="urn:ietf:params:xml:ns:yang:ietf-netconf-monitoring">'

VALID_DATA = (
    '<netconf-state xmlns="urn:ietf:params:xml:ns:yang:ietf-netconf-monitoring">'
    "<schemas><schema/></schemas></netconf-state>"
)

OUTPUT = """{"netconf-state": \
{"@xmlns": "urn:ietf:params:xml:ns:yang:ietf-netconf-monitoring", "schemas": {"schema": null}}}"""


class TestFromXml(unittest.TestCase):
    def setUp(self):
        pass

    def test_invalid_data(self):
        """Check passing invalid argspec"""

        # missing required arguments
        args = ["", INVALID_DATA, "xmltodict"]
        kwargs = {}
        with self.assertRaises(AnsibleError) as error:
            _from_xml(*args, **kwargs)
        self.assertIn(
            "Error when using plugin 'from_xml': Input Xml is not valid",
            str(error.exception),
        )

    def test_valid_data(self):
        """Check passing valid data as per criteria"""
        self.maxDiff = None
        args = ["", VALID_DATA, "xmltodict"]
        result = _from_xml(*args)
        self.assertEqual(result, OUTPUT)

    def test_args(self):
        """Check passing invalid argspec"""

        # missing required arguments
        args = []
        kwargs = {}
        with self.assertRaises(AnsibleFilterError) as error:
            _from_xml(*args, **kwargs)
        self.assertIn("missing required arguments: data", str(error.exception))

    def test_invalid_engine(self):
        """Check passing invalid argspec"""

        # missing required arguments
        args = ["", INVALID_DATA, "test"]
        kwargs = {}
        with self.assertRaises(AnsibleError) as error:
            _from_xml(*args, **kwargs)
        self.assertIn("engine: test is not supported", str(error.exception))



# -*- coding: utf-8 -*-
# Copyright 2020 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

import unittest
from ansible.playbook.task import Task
from ansible.template import Templar
from ansible.errors import AnsibleActionFail

from ansible_collections.ansible.utils.plugins.action.validate import (
    ActionModule,
)

try:
    from unittest.mock import MagicMock  # pylint:disable=syntax-error
except ImportError:
    from mock import MagicMock


DATA = {
    "GigabitEthernet0/0/0/0": {
        "auto_negotiate": False,
        "counters": {
            "in_crc_errors": 0,
            "in_errors": 0,
            "rate": {"in_rate": 0, "out_rate": 0},
        },
        "description": "configured using Ansible",
        "duplex_mode": "full",
        "enabled": True,
        "line_protocol": "up",
        "mtu": 1514,
        "oper_status": "down",
        "type": "GigabitEthernet",
    },
    "GigabitEthernet0/0/0/1": {
        "auto_negotiate": False,
        "counters": {
            "in_crc_errors": 10,
            "in_errors": 0,
            "rate": {"in_rate": 0, "out_rate": 0},
        },
        "description": "# interface is configures with Ansible",
        "duplex_mode": "full",
        "enabled": False,
        "line_protocol": "up",
        "mtu": 1514,
        "oper_status": "up",
        "type": "GigabitEthernet",
    },
}

CRITERIA_CRC_ERROR_CHECK = {
    "type": "object",
    "patternProperties": {
        "^.*": {
            "type": "object",
            "properties": {
                "counters": {
                    "properties": {
                        "in_crc_errors": {"type": "number", "maximum": 0}
                    }
                }
            },
        }
    },
}

CRITERIA_ENABLED_CHECK = {
    "type": "object",
    "patternProperties": {
        "^.*": {"type": "object", "properties": {"enabled": {"enum": [True]}}}
    },
}

CRITERIA_OPER_STATUS_UP_CHECK = {
    "type": "object",
    "patternProperties": {
        "^.*": {
            "type": "object",
            "properties": {"oper_status": {"type": "string", "pattern": "up"}},
        }
    },
}

CRITERIA_IN_RATE_CHECK = {
    "type": "object",
    "patternProperties": {
        "^.*": {
            "type": "object",
            "properties": {
                "counters": {
                    "properties": {
                        "rate": {
                            "properties": {
                                "in_rate": {"type": "number", "maximum": 0}
                            }
                        }
                    }
                }
            },
        }
    },
}


class TestValidate(unittest.TestCase):
    def setUp(self):
        task = MagicMock(Task)
        play_context = MagicMock()
        play_context.check_mode = False
        connection = MagicMock()
        fake_loader = {}
        templar = Templar(loader=fake_loader)
        self._plugin = ActionModule(
            task=task,
            connection=connection,
            play_context=play_context,
            loader=fake_loader,
            templar=templar,
            shared_loader_obj=None,
        )
        self._plugin._task.action = "validate"

    def test_invalid_argspec(self):
        """Check passing invalid argspec"""

        # missing required arguments
        self._plugin._task.args = {"engine": "ansible.utils.jsonschema"}
        result = self._plugin.run(task_vars=None)
        msg = "missing required arguments criteria data"
        for word in msg.split():
            self.assertIn(word, result["errors"])

        # invalid engine option value
        self._plugin._task.args = {
            "engine": "ansible.utils.sample",
            "data": DATA,
            "criteria": CRITERIA_OPER_STATUS_UP_CHECK,
        }
        result = self._plugin.run(task_vars=None)
        self.assertIn(
            "For engine 'ansible.utils.sample' error loading the corresponding validate plugin",
            result["msg"],
        )

        # invalid data option value
        self._plugin._task.args = {
            "engine": "ansible.utils.jsonschema",
            "data": "invalid data",
            "criteria": CRITERIA_OPER_STATUS_UP_CHECK,
        }

        with self.assertRaises(AnsibleActionFail) as error:
            self._plugin.run(task_vars=None)
        self.assertIn("'data' option value is invalid", str(error.exception))

        # invalid criteria option value
        self._plugin._task.args = {
            "engine": "ansible.utils.jsonschema",
            "data": DATA,
            "criteria": "invalid criteria",
        }

        with self.assertRaises(AnsibleActionFail) as error:
            self._plugin.run(task_vars=None)
        self.assertIn(
            "'criteria' option value is invalid", str(error.exception)
        )

    def test_invalid_validate_plugin_config_options(self):
        """Check passing invalid validate plugin options"""

        self._plugin._task.args = {
            "engine": "ansible.utils.jsonschema",
            "data": DATA,
            "criteria": CRITERIA_IN_RATE_CHECK,
        }

        result = self._plugin.run(
            task_vars={"ansible_validate_jsonschema_draft": "draft0"}
        )
        self.assertIn(
            "value of draft must be one of: draft3, draft4, draft6, draft7, got: draft0",
            result["msg"],
        )

    def test_invalid_data(self):
        """Check passing invalid data as per criteria"""

        self._plugin._task.args = {
            "engine": "ansible.utils.jsonschema",
            "data": DATA,
            "criteria": [
                CRITERIA_CRC_ERROR_CHECK,
                CRITERIA_ENABLED_CHECK,
                CRITERIA_OPER_STATUS_UP_CHECK,
            ],
        }

        result = self._plugin.run(task_vars=None)
        self.assertIn(
            "patternProperties.^.*.properties.counters.properties.in_crc_errors.maximum",
            result["msg"],
        )
        self.assertIn(
            "patternProperties.^.*.properties.enabled.enum", result["msg"]
        )
        self.assertIn(
            "'patternProperties.^.*.properties.oper_status.pattern",
            result["msg"],
        )

    def test_valid_data(self):
        """Check passing valid data as per criteria"""

        self._plugin._task.args = {
            "engine": "ansible.utils.jsonschema",
            "data": DATA,
            "criteria": CRITERIA_IN_RATE_CHECK,
        }

        result = self._plugin.run(task_vars=None)
        self.assertIn("all checks passed", result["msg"])



#
# -*- coding: utf-8 -*-
# Copyright 2021 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
#

"""
The to_xml plugin
"""
from __future__ import absolute_import, division, print_function

__metaclass__ = type

from ansible.errors import AnsibleFilterError

try:
    import xmltodict

    HAS_XMLTODICT = True
except ImportError:
    HAS_XMLTODICT = False


def _raise_error(msg):
    """Raise an error message, prepend with filter name

    :param msg: The message
    :type msg: str
    :raises: AnsibleError
    """
    error = "Error when using plugin 'to_xml': {msg}".format(msg=msg)
    raise AnsibleFilterError(error)


def to_xml(data, engine):
    """Convert data which is in json to xml"

    :param data: The data passed in (data|to_xml(...))
    :type data: xml
    :param engine: Conversion library default=xmltodict
    """
    if engine == "xmltodict":
        if not HAS_XMLTODICT:
            _raise_error("Missing required library xmltodict")
        try:
            res = xmltodict.unparse(data, pretty=True)
        except Exception:
            _raise_error("Input json is not valid")
        return res
    else:
        error = "engine: {engine} is not supported ".format(engine=engine)
        _raise_error(error)



# -*- coding: utf-8 -*-
# Copyright 2021 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

"""
The utils file for all netaddr tests
"""

from __future__ import absolute_import, division, print_function

__metaclass__ = type

from ansible.errors import AnsibleError
from ansible.module_utils.basic import missing_required_lib
from ansible.module_utils.six import ensure_text
from functools import wraps
from ansible_collections.ansible.utils.plugins.module_utils.common.argspec_validate import (
    check_argspec,
)

try:
    import ipaddress

    HAS_IPADDRESS = True
except ImportError:
    HAS_IPADDRESS = False


def ip_network(ip):
    """ PY2 compat shim, PY2 requires unicode
    """

    if not HAS_IPADDRESS:
        raise AnsibleError(missing_required_lib("ipaddress"))

    return ipaddress.ip_network(ensure_text(ip))


def ip_address(ip):
    """ PY2 compat shim, PY2 requires unicode
    """

    if not HAS_IPADDRESS:
        raise AnsibleError(missing_required_lib("ipaddress"))

    return ipaddress.ip_address(ensure_text(ip))


def _need_ipaddress(func):
    @wraps(func)
    def wrapper(*args, **kwargs):
        if not HAS_IPADDRESS:
            raise AnsibleError(missing_required_lib("ipaddress"))
        return func(*args, **kwargs)

    return wrapper


def _is_subnet_of(network_a, network_b):
    try:
        if network_a._version != network_b._version:
            return False
        return (
            network_b.network_address <= network_a.network_address
            and network_b.broadcast_address >= network_a.broadcast_address
        )
    except Exception:
        return False


def _validate_args(plugin, doc, params):
    """ argspec validator utility function
    """

    valid, argspec_result, updated_params = check_argspec(
        doc, plugin + " test", **params
    )

    if not valid:
        raise AnsibleError(
            "{argspec_result} with errors: {argspec_errors}".format(
                argspec_result=argspec_result.get("msg"),
                argspec_errors=argspec_result.get("errors"),
            )
        )



"""
ttp parser

This is the ttp parser for use with the cli_parse module and action plugin
https://github.com/dmulyalin/ttp
"""
from __future__ import absolute_import, division, print_function

__metaclass__ = type

DOCUMENTATION = """
    author: Bradley Thornton (@cidrblock)
    name: ttp
    short_description: Define configurable options for C(ttp) sub-plugin of M(ansible.utils.cli_parse) module
    description:
    - This plugin documentation provides the configurable options that can be passed
      to the M(ansible.utils.cli_parse) plugins when I(ansible.utils.ttp) is used as a value for
      I(name) option.
    version_added: 1.0.0
"""

EXAMPLES = r"""
- name: "Run command and parse with textfsm"
  ansible.utils.cli_parse:
    command: "show version"
    parser:
      name: ansible.utils.ttp
  register: nxos_ttp_command

- name: "Pass text and command"
  ansible.utils.cli_parse:
    text: "{{ lookup('ansible.builtin.file', '/home/user/files/nxos_show_version.txt') }}"
    parser:
      name: ansible.utils.textfsm
      template_path: "/home/user/templates/nxos_show_version.ttp"
  register: nxos_ttp_text
"""

import os

from ansible.module_utils._text import to_native
from ansible.module_utils.basic import missing_required_lib
from ansible_collections.ansible.utils.plugins.plugin_utils.base.cli_parser import (
    CliParserBase,
)

try:
    from ttp import ttp

    HAS_TTP = True
except ImportError:
    HAS_TTP = False


class CliParser(CliParserBase):
    """ The ttp parser class
    Convert raw text to structured data using ttp
    """

    DEFAULT_TEMPLATE_EXTENSION = "ttp"
    PROVIDE_TEMPLATE_CONTENTS = False

    @staticmethod
    def _check_reqs():
        """ Check the prerequisites for the ttp parser

        :return dict: A dict with errors or a template_path
        """
        errors = []

        if not HAS_TTP:
            errors.append(missing_required_lib("ttp"))

        return {"errors": errors}

    def parse(self, *_args, **_kwargs):
        """ Std entry point for a cli_parse parse execution

        :return: Errors or parsed text as structured data
        :rtype: dict

        :example:

        The parse function of a parser should return a dict:
        {"errors": [a list of errors]}
        or
        {"parsed": obj}
        """
        cli_output = to_native(
            self._task_args.get("text"), errors="surrogate_then_replace"
        )
        res = self._check_reqs()
        if res.get("errors"):
            return {"errors": res.get("errors")}

        template_path = to_native(
            self._task_args.get("parser").get("template_path"),
            errors="surrogate_then_replace",
        )
        if template_path and not os.path.isfile(template_path):
            return {
                "errors": "error while reading template_path file {file}".format(
                    file=template_path
                )
            }

        try:
            parser_param = self._task_args.get("parser")
            vars = (
                parser_param.get("vars", {}).get("ttp_vars", {})
                if parser_param.get("vars")
                else {}
            )
            kwargs = (
                parser_param.get("vars", {}).get("ttp_init", {})
                if parser_param.get("vars")
                else {}
            )
            parser = ttp(
                data=cli_output, template=template_path, vars=vars, **kwargs
            )
            parser.parse(one=True)
            ttp_results = (
                parser_param.get("vars", {}).get("ttp_results", {})
                if parser_param.get("vars")
                else {}
            )
            results = parser.result(**ttp_results)
        except Exception as exc:
            msg = "Template Text Parser returned an error while parsing. Error: {err}"
            return {"errors": [msg.format(err=to_native(exc))]}
        return {"parsed": results}



# -*- coding: utf-8 -*-
# Copyright 2020 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

import unittest
from ansible.errors import AnsibleError
from ansible.errors import AnsibleFilterError
from ansible_collections.ansible.utils.plugins.filter.to_xml import _to_xml

INVALID_DATA = '<netconf-state xmlns="urn:ietf:params:xml:ns:yang:ietf-netconf-monitoring">'

VALID_DATA = {
    "interface-configurations": {
        "@xmlns": "http://cisco.com/ns/yang/Cisco-IOS-XR-ifmgr-cfg"
    }
}

OUTPUT = """<?xml version="1.0" encoding="utf-8"?>
<interface-configurations xmlns="http://cisco.com/ns/yang/Cisco-IOS-XR-ifmgr-cfg"></interface-configurations>"""


class TestToXml(unittest.TestCase):
    def setUp(self):
        pass

    def test_invalid_data(self):
        """Check passing invalid argspec"""

        # missing required arguments
        args = ["", INVALID_DATA, "xmltodict"]
        kwargs = {}
        with self.assertRaises(AnsibleError) as error:
            _to_xml(*args, **kwargs)
        self.assertIn(
            "we were unable to convert to dict", str(error.exception)
        )

    def test_valid_data(self):
        """Check passing valid data as per criteria"""
        self.maxDiff = None
        args = ["", VALID_DATA, "xmltodict"]
        result = _to_xml(*args)
        self.assertEqual(result, OUTPUT)

    def test_args(self):
        """Check passing invalid argspec"""

        # missing required arguments
        args = []
        kwargs = {}
        with self.assertRaises(AnsibleFilterError) as error:
            _to_xml(*args, **kwargs)
        self.assertIn("missing required arguments: data", str(error.exception))

    def test_invalid_engine(self):
        """Check passing invalid argspec"""

        # missing required arguments
        args = ["", VALID_DATA, "test"]
        kwargs = {}
        with self.assertRaises(AnsibleError) as error:
            _to_xml(*args, **kwargs)
        self.assertIn("engine: test is not supported", str(error.exception))



# -*- coding: utf-8 -*-
# Copyright 2020 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type


class FactDiffBase:
    def __init__(self, task_args, task_vars, debug):
        self._debug = debug
        self._task_args = task_args
        self._task_vars = task_vars



# -*- coding: utf-8 -*-
# Copyright 2021 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

"""
Unit test file for usable_range filter plugin
"""

from __future__ import absolute_import, division, print_function

__metaclass__ = type

import unittest
from ansible.errors import AnsibleError
from ansible_collections.ansible.utils.plugins.filter.usable_range import (
    _usable_range,
)

INVALID_DATA_1 = [
    "helloworld",
    "192.0.2.0/23/24",
    "::/20/30",
    "10.0.0.0/322",
    "2001:db8:abcd:0012::0/129",
]
INVALID_DATA_2 = ["192.168.1.25/24", "2001:db8:abcd:12::2/126"]

VALID_DATA = [
    "10.0.0.8/30",
    "192.0.2.0/28",
    "2001:db8:abcd:0012::0/126",
    "2001:DB8:ABCD:12::",
]

VALID_OUTPUT_1 = {
    "number_of_ips": 4,
    "usable_ips": ["10.0.0.8", "10.0.0.9", "10.0.0.10", "10.0.0.11"],
}
VALID_OUTPUT_2 = {
    "number_of_ips": 16,
    "usable_ips": [
        "192.0.2.0",
        "192.0.2.1",
        "192.0.2.2",
        "192.0.2.3",
        "192.0.2.4",
        "192.0.2.5",
        "192.0.2.6",
        "192.0.2.7",
        "192.0.2.8",
        "192.0.2.9",
        "192.0.2.10",
        "192.0.2.11",
        "192.0.2.12",
        "192.0.2.13",
        "192.0.2.14",
        "192.0.2.15",
    ],
}

VALID_OUTPUT_3 = {
    "number_of_ips": 4,
    "usable_ips": [
        "2001:db8:abcd:12::",
        "2001:db8:abcd:12::1",
        "2001:db8:abcd:12::2",
        "2001:db8:abcd:12::3",
    ],
}
VALID_OUTPUT_4 = {"number_of_ips": 1, "usable_ips": ["2001:db8:abcd:12::"]}


class TestUsableRange(unittest.TestCase):
    def setUp(self):
        pass

    def test_missing_data(self):
        """Check passing missing argspec"""

        # missing required arguments
        ip = ""
        with self.assertRaises(AnsibleError) as error:
            _usable_range(ip)
        self.assertIn(
            "does not appear to be an IPv4 or IPv6 network",
            str(error.exception),
        )

    def test_invalid_data(self):
        """Check passing invalid argspec"""

        # invalid required arguments

        for invalid_data in INVALID_DATA_1:
            with self.assertRaises(AnsibleError) as error:
                _usable_range(invalid_data)
            self.assertIn(
                "does not appear to be an IPv4 or IPv6 network",
                str(error.exception),
            )

        for invalid_data in INVALID_DATA_2:
            with self.assertRaises(AnsibleError) as error:
                _usable_range(invalid_data)
            self.assertIn("has host bits set", str(error.exception))

    def test_valid_data(self):
        """Check passing valid data as per criteria"""

        ip = VALID_DATA[0]
        result = _usable_range(ip)
        self.assertEqual(result, VALID_OUTPUT_1)

        ip = VALID_DATA[1]
        result = _usable_range(ip)
        self.assertEqual(result, VALID_OUTPUT_2)

        ip = VALID_DATA[2]
        result = _usable_range(ip)
        self.assertEqual(result, VALID_OUTPUT_3)

        ip = VALID_DATA[3]
        result = _usable_range(ip)
        self.assertEqual(result, VALID_OUTPUT_4)



# -*- coding: utf-8 -*-
# Copyright 2020 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)


"""
flatten a complex object to dot bracket notation
"""
from __future__ import absolute_import, division, print_function

__metaclass__ = type


DOCUMENTATION = """
    name: to_paths
    author: Bradley Thornton (@cidrblock)
    version_added: "1.0.0"
    short_description: Flatten a complex object into a dictionary of paths and values
    description:
        - Flatten a complex object into a dictionary of paths and values.
        - Paths are dot delimited whenever possible.
        - Brackets are used for list indices and keys that contain special characters.
        - B(to_paths) is also available as a B(lookup plugin) for convenience.
        - Using the parameters below- C(var|ansible.utils.to_paths(prepend, wantlist))
    options:
      var:
        description:
        - The value of I(var) will be will be used.
        - This option represents the value that is passed to the filter plugin in pipe format.
        - For example C(config_data|ansible.utils.to_paths()), in this case C(config_data) represents this option.
        type: raw
        required: True
      prepend:
        description: Prepend each path entry. Useful to add the initial I(var) name.
        type: str
        required: False
      wantlist:
        description:
        - If set to C(True), the return value will always be a list.
        type: bool

    notes:
"""

EXAMPLES = r"""

#### Simple examples

- ansible.builtin.set_fact:
    a:
      b:
        c:
          d:
          - 0
          - 1
          e:
          - True
          - False

- ansible.builtin.set_fact:
    paths: "{{ a|ansible.utils.to_paths }}"

# TASK [ansible.builtin.set_fact] ********************************************
# ok: [nxos101] => changed=false
#   ansible_facts:
#     paths:
#       b.c.d[0]: 0
#       b.c.d[1]: 1
#       b.c.e[0]: True
#       b.c.e[1]: False

- name: Use prepend to add the initial variable name
  ansible.builtin.set_fact:
    paths: "{{ a|ansible.utils.to_paths(prepend='a') }}"

# TASK [Use prepend to add the initial variable name] **************************
# ok: [nxos101] => changed=false
#   ansible_facts:
#     paths:
#       a.b.c.d[0]: 0
#       a.b.c.d[1]: 1
#       a.b.c.e[0]: True
#       a.b.c.e[1]: False


#### Using a complex object

- name: Make an API call
  uri:
    url: "https://nxos101/restconf/data/openconfig-interfaces:interfaces"
    headers:
      accept: "application/yang.data+json"
    url_password: password
    url_username: admin
    validate_certs: False
  register: result
  delegate_to: localhost

- name: Flatten the complex object
  ansible.builtin.set_fact:
    paths: "{{ result.json|ansible.utils.to_paths }}"

# TASK [Flatten the complex object] ******************************************
# ok: [nxos101] => changed=false
#   ansible_facts:
#     paths:
#       interfaces.interface[0].config.enabled: 'true'
#       interfaces.interface[0].config.mtu: '1500'
#       interfaces.interface[0].config.name: eth1/71
#       interfaces.interface[0].config.type: ethernetCsmacd
#       interfaces.interface[0].ethernet.config['auto-negotiate']: 'true'
#       interfaces.interface[0].ethernet.state.counters['in-crc-errors']: '0'
#       interfaces.interface[0].ethernet.state.counters['in-fragment-frames']: '0'
#       interfaces.interface[0].ethernet.state.counters['in-jabber-frames']: '0'
#       interfaces.interface[0].ethernet.state.counters['in-mac-control-frames']: '0'
#       <...>
"""

from ansible.errors import AnsibleFilterError
from ansible_collections.ansible.utils.plugins.module_utils.common.to_paths import (
    to_paths,
)
from ansible_collections.ansible.utils.plugins.module_utils.common.argspec_validate import (
    AnsibleArgSpecValidator,
)


def _to_paths(*args, **kwargs):
    """Flatten a complex object into a dictionary of paths and values."""
    keys = ["var", "prepend", "wantlist"]
    data = dict(zip(keys, args))
    data.update(kwargs)
    aav = AnsibleArgSpecValidator(
        data=data, schema=DOCUMENTATION, name="to_paths"
    )
    valid, errors, updated_data = aav.validate()
    if not valid:
        raise AnsibleFilterError(errors)
    return to_paths(**updated_data)


class FilterModule(object):
    """ path filters """

    def filters(self):
        return {"to_paths": _to_paths}



# -*- coding: utf-8 -*-
# Copyright 2020 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)


from __future__ import absolute_import, division, print_function

__metaclass__ = type

import unittest
from ansible_collections.ansible.utils.plugins.module_utils.common.utils import (
    dict_merge,
)


class TestDict_merge(unittest.TestCase):
    def test_not_dict_base(self):
        base = [0]
        other = {"a": "b"}
        expected = "must be of type <dict>"
        with self.assertRaises(Exception) as exc:
            dict_merge(base, other)
        self.assertIn(expected, str(exc.exception))

    def test_not_dict_other(self):
        base = {"a": "b"}
        other = [0]
        expected = "must be of type <dict>"
        with self.assertRaises(Exception) as exc:
            dict_merge(base, other)
        self.assertIn(expected, str(exc.exception))

    def test_simple(self):
        base = {"a": "b"}
        other = {"c": "d"}
        expected = {"a": "b", "c": "d"}
        result = dict_merge(base, other)
        self.assertEqual(result, expected)

    def test_simple_other_is_string(self):
        base = {"a": "b"}
        other = {"a": "c"}
        expected = {"a": "c"}
        result = dict_merge(base, other)
        self.assertEqual(result, expected)

    def test_simple_other_is_none(self):
        base = {"a": "b"}
        other = {"a": None}
        expected = {"a": None}
        result = dict_merge(base, other)
        self.assertEqual(result, expected)

    def test_list_value_same(self):
        base = {"a": [2, 1, 0]}
        other = {"a": [0, 1, 2]}
        expected = {"a": [0, 1, 2]}
        result = dict_merge(base, other)
        self.assertEqual(result, expected)

    def test_list_value_combine(self):
        base = {"a": [2, 1, 0]}
        other = {"a": [0, 3]}
        expected = {"a": [2, 1, 0, 3]}
        result = dict_merge(base, other)
        self.assertEqual(result, expected)

    def test_list_missing_from_other(self):
        base = {"a": [2, 1, 0]}
        other = {"b": [2, 1, 0]}
        expected = {"a": [2, 1, 0], "b": [2, 1, 0]}
        result = dict_merge(base, other)
        self.assertEqual(result, expected)

    def test_list_other_none(self):
        base = {"a": [2, 1, 0]}
        other = {"a": None}
        expected = {"a": None}
        result = dict_merge(base, other)
        self.assertEqual(result, expected)

    # modified dict merge from netcommon so this works
    def test_list_other_dict(self):
        base = {"a": [2, 1, 0]}
        other = {"a": {"b": "c"}}
        expected = {"a": {"b": "c"}}
        result = dict_merge(base, other)
        self.assertEqual(result, expected)

    # modified dict merge from netcommon so this works
    def test_list_other_string(self):
        base = {"a": [2, 1, 0]}
        other = {"a": "xyz"}
        expected = {"a": "xyz"}
        result = dict_merge(base, other)
        self.assertEqual(result, expected)

    def test_dict_of_dict_merged(self):
        base = {"a": {"b": 0}}
        other = {"a": {"c": 1}}
        expected = {"a": {"b": 0, "c": 1}}
        result = dict_merge(base, other)
        self.assertEqual(result, expected)

    def test_dict_of_dict_replaced(self):
        base = {"a": {"b": 0}}
        other = {"a": {"b": 1}}
        expected = {"a": {"b": 1}}
        result = dict_merge(base, other)
        self.assertEqual(result, expected)

    def test_dict_of_dict_replaced_other_none(self):
        base = {"a": {"b": 0}}
        other = {"a": None}
        expected = {"a": None}
        result = dict_merge(base, other)
        self.assertEqual(result, expected)

    def test_dict_of_dict_replaced_other_string(self):
        base = {"a": {"b": 0}}
        other = {"a": "xyz"}
        expected = {"a": "xyz"}
        result = dict_merge(base, other)
        self.assertEqual(result, expected)

    def test_dict_of_dict_replaced_other_missing(self):
        base = {"a": {"b": 0}}
        other = {"c": 1}
        expected = {"a": {"b": 0}, "c": 1}
        result = dict_merge(base, other)
        self.assertEqual(result, expected)

    def test_not_list_or_dict_different(self):
        base = {"a": 0}
        other = {"a": 1}
        expected = {"a": 1}
        result = dict_merge(base, other)
        self.assertEqual(result, expected)

    def test_not_list_or_dict_same(self):
        base = {"a": 0}
        other = {"a": 0}
        expected = {"a": 0}
        result = dict_merge(base, other)
        self.assertEqual(result, expected)



DOCUMENTATION = """
module: test
author: Bradley Thornton (@cidrblock)
short_description: Short description here
description:
- A longer description here
version_added: 0.0.0
options:
    param_str:
        type: str
        description:
        - A string param
        required: True
    params_bool:
        type: bool
        description:
        - A bool param
    params_dict:
        type: dict
        description:
        - A dict param
        suboptions:
            subo_str:
                type: str
                description:
                - A string suboption
            subo_list:
                type: list
                description:
                - A list suboption
            subo_dict:
                type: dict
                description:
                - A dict suboption
    param_default:
        type: bool
        default: True
"""



# -*- coding: utf-8 -*-
# Copyright 2021 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

"""
Test plugin file for netaddr tests: mac
"""

from __future__ import absolute_import, division, print_function
from ansible_collections.ansible.utils.plugins.plugin_utils.base.ipaddress_utils import (
    _validate_args,
)
import re

__metaclass__ = type

DOCUMENTATION = """
    name: mac
    author: Priyam Sahoo (@priyamsahoo)
    version_added: "2.2.0"
    short_description: Test if something appears to be a valid MAC address
    description:
        - This plugin checks if the provided value is a valid MAC address that follows the industry level standards
    options:
        mac:
            description:
            - A string that represents the value against which the test is going to be performed
            - For example:
                - "02:16:3e:e4:16:f3"
                - "02-16-3e-e4-16-f3"
                - "0216.3ee4.16f3"
                - "02163ee416f3"
            type: str
            required: True
    notes:
"""

EXAMPLES = r"""

- name: Check if 02:16:3e:e4:16:f3 is a valid MAC address
  ansible.builtin.set_fact:
    data: "{{ '02:16:3e:e4:16:f3' is ansible.utils.mac }}"

# TASK [Check if 02:16:3e:e4:16:f3 is a valid MAC address] ********************
# ok: [localhost] => {
#     "ansible_facts": {
#         "data": true
#     },
#     "changed": false
# }

- name: Check if 02-16-3e-e4-16-f3 is a valid MAC address
  ansible.builtin.set_fact:
    data: "{{ '02-16-3e-e4-16-f3' is ansible.utils.mac }}"

# TASK [Check if 02-16-3e-e4-16-f3 is a valid MAC address] ********************
# ok: [localhost] => {
#     "ansible_facts": {
#         "data": true
#     },
#     "changed": false
# }

- name: Check if 0216.3ee4.16f3 is a valid MAC address
  ansible.builtin.set_fact:
    data: "{{ '0216.3ee4.16f3' is ansible.utils.mac }}"

# TASK [Check if 0216.3ee4.16f3 is a valid MAC address] ***********************
# ok: [localhost] => {
#     "ansible_facts": {
#         "data": true
#     },
#     "changed": false
# }

- name: Check if 02163ee416f3 is a valid MAC address
  ansible.builtin.set_fact:
    data: "{{ '02163ee416f3' is ansible.utils.mac }}"

# TASK [Check if 02163ee416f3 is a valid MAC address] *************************
# ok: [localhost] => {
#     "ansible_facts": {
#         "data": true
#     },
#     "changed": false
# }

- name: Check if helloworld is not a valid MAC address
  ansible.builtin.set_fact:
    data: "{{ 'helloworld' is not ansible.utils.mac }}"

# TASK [Check if helloworld is not a valid MAC address] ***********************
# ok: [localhost] => {
#     "ansible_facts": {
#         "data": true
#     },
#     "changed": false
# }

"""

RETURN = """
  data:
    description:
      - If jinja test satisfies plugin expression C(true)
      - If jinja test does not satisfy plugin expression C(false)
"""


def _mac(mac):
    """ Test if something appears to be a valid mac address """

    params = {"mac": mac}
    _validate_args("mac", DOCUMENTATION, params)

    # IEEE EUI-48 upper and lower, commom unix
    re1 = r"^(?i)([0-9a-f]{2}[:-]){5}[0-9a-f]{2}$"
    # Cisco triple hextex
    re2 = r"^(?i)([0-9a-f]{4}\.[0-9a-f]{4}\.[0-9a-f]{4})$"
    # Bare
    re3 = r"^(?i)[0-9a-f]{12}$"
    regex = "{re1}|{re2}|{re3}".format(re1=re1, re2=re2, re3=re3)
    return bool(re.match(regex, mac))


class TestModule(object):
    """ network jinja test"""

    test_map = {"mac": _mac}

    def tests(self):
        return self.test_map



# -*- coding: utf-8 -*-
# Copyright 2021 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

"""
Test plugin file for netaddr tests: in_network
"""

from __future__ import absolute_import, division, print_function
from ansible_collections.ansible.utils.plugins.plugin_utils.base.ipaddress_utils import (
    ip_network,
    _is_subnet_of,
    _need_ipaddress,
)
from ansible_collections.ansible.utils.plugins.plugin_utils.base.ipaddress_utils import (
    _validate_args,
)

__metaclass__ = type

DOCUMENTATION = """
    name: in_network
    author: Priyam Sahoo (@priyamsahoo)
    version_added: "2.2.0"
    short_description: Test if IP address falls in the network
    description:
        - This plugin checks if the provided IP address belongs to the provided network
    options:
        ip:
            description:
            - A string that represents an IP address
            - For example: "10.1.1.1"
            type: str
            required: True
        network:
            description:
            - A string that represents the network address in CIDR form
            - For example: "10.0.0.0/8"
            type: str
            required: True
    notes:
"""

EXAMPLES = r"""

#### Simple examples

- name: Check if 10.1.1.1 is in 10.0.0.0/8
  ansible.builtin.set_fact:
    data: "{{ '10.1.1.1' is ansible.utils.in_network '10.0.0.0/8' }}"

# TASK [Check if 10.1.1.1 is in 10.0.0.0/8] ***********************************
# ok: [localhost] => {
#     "ansible_facts": {
#         "data": true
#     },
#     "changed": false
# }

- name: Check if 10.1.1.1 is not in 192.168.1.0/24
  ansible.builtin.set_fact:
    data: "{{ '10.1.1.1' is not ansible.utils.in_network '192.168.1.0/24' }}"

# TASK [Check if 10.1.1.1 is not in 192.168.1.0/24] ****************************
# ok: [localhost] => {
#     "ansible_facts": {
#         "data": true
#     },
#     "changed": false
# }

- name: Check if 2001:db8:a::123 is in 2001:db8:a::/64
  ansible.builtin.set_fact:
    data: "{{ '2001:db8:a::123' is ansible.utils.in_network '2001:db8:a::/64' }}"

# TASK [Check if 2001:db8:a::123 is in 2001:db8:a::/64] ****************************
# task path: /home/prsahoo/playbooks/collections/localhost_test/utils_in_network.yml:16
# ok: [localhost] => {
#     "ansible_facts": {
#         "data": true
#     },
#     "changed": false
# }

- name: Check if 2001:db8:a::123 is not in 10.0.0.0/8
  ansible.builtin.set_fact:
    data: "{{ '2001:db8:a::123' is not ansible.utils.in_network '10.0.0.0/8' }}"

# TASK [Check if 2001:db8:a::123 is not in 10.0.0.0/8] *********************************
# task path: /home/prsahoo/playbooks/collections/localhost_test/utils_in_network.yml:20
# ok: [localhost] => {
#     "ansible_facts": {
#         "data": true
#     },
#     "changed": false
# }

"""

RETURN = """
  data:
    description:
      - If jinja test satisfies plugin expression C(true)
      - If jinja test does not satisfy plugin expression C(false)
"""


@_need_ipaddress
def _in_network(ip, network):
    """Test if an address or network is in a network"""

    params = {"ip": ip, "network": network}
    _validate_args("in_network", DOCUMENTATION, params)

    try:
        return _is_subnet_of(ip_network(ip), ip_network(network))
    except Exception:
        return False


class TestModule(object):
    """ network jinja test"""

    test_map = {"in_network": _in_network}

    def tests(self):
        return self.test_map



# -*- coding: utf-8 -*-
# Copyright 2021 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

"""
Unit test file for netaddr test plugin: in_one_network
"""

from __future__ import absolute_import, division, print_function

__metaclass__ = type

import unittest
from ansible.errors import AnsibleError
from ansible_collections.ansible.utils.plugins.test.in_one_network import (
    _in_one_network,
)


class TestInOneNetwork(unittest.TestCase):
    def setUp(self):
        pass

    def test_invalid_data(self):
        """Check passing invalid argspec"""

        # invalid argument
        with self.assertRaises(AnsibleError) as error:
            _in_one_network(
                ip="10.1.1.1",
                networks={
                    "name": "networks",
                    "value": ["10.0.0.0/8", "192.168.1.0/24"],
                },
            )
        self.assertIn("unable to convert to list", str(error.exception))

    def test_valid_data(self):
        """Check passing valid data as per criteria"""

        result = _in_one_network(
            ip="10.1.1.1", networks=["10.0.0.0/8", "192.168.1.0/24"]
        )
        self.assertEqual(result, True)

        result = _in_one_network(
            ip="8.8.8.8", networks=["10.0.0.0/8", "10.1.1.0/24"]
        )
        self.assertEqual(result, False)



# -*- coding: utf-8 -*-
# Copyright 2021 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

"""
Test plugin file for netaddr tests: resolvable
"""
from __future__ import absolute_import, division, print_function
from ansible_collections.ansible.utils.plugins.plugin_utils.base.ipaddress_utils import (
    _need_ipaddress,
    _validate_args,
)

import socket

try:
    import ipaddress

    HAS_IPADDRESS = True
except ImportError:
    HAS_IPADDRESS = False

__metaclass__ = type

DOCUMENTATION = """
    name: resolvable
    author: Priyam Sahoo (@priyamsahoo)
    version_added: "2.2.0"
    short_description: Test if an IP or name can be resolved via /etc/hosts or DNS
    description:
        - This plugin checks if the provided IP address of host name can be resolved using /etc/hosts or DNS
    options:
        host:
            description:
            - A string that represents the IP address or the host name
            - For example:
                - "docs.ansible.com"
                - 127.0.0.1
                - ::1
            type: str
            required: True
    notes:
"""

EXAMPLES = r"""

#### Simple examples

- name: Check if docs.ansible.com is resolvable or not
  ansible.builtin.set_fact:
    data: "{{ 'docs.ansible.com' is ansible.utils.resolvable }}"

# TASK [Check if docs.ansible.com is resolvable or not] **********************************
# ok: [localhost] => {
#     "ansible_facts": {
#         "data": true
#     },
#     "changed": false
# }

- name: Set host name variables
  ansible.builtin.set_fact:
    good_name: www.google.com
    bad_name: foo.google.com

- name: Assert good_name's resolvability
  assert:
    that: "{{ 'www.google.com' is ansible.utils.resolvable }}"

- name: Assert bad_name's resolvability
  assert:
    that: "{{ 'foo.google.com' is not ansible.utils.resolvable }}"

# TASK [Assert good_name's resolvability] ************************************************
# ok: [localhost] => {
#     "changed": false,
#     "msg": "All assertions passed"
# }

# TASK [Assert bad_name's resolvability] *************************************************
# ok: [localhost] => {
#     "changed": false,
#     "msg": "All assertions passed"
# }

- name: Set ip variables
  ansible.builtin.set_fact:
    ipv4_localhost: "127.0.0.1"
    ipv6_localhost: "::1"

- name: Assert ipv4_localhost's resolvability
  assert:
    that: "{{ ipv4_localhost is ansible.utils.resolvable }}"

- name: Assert ipv6_localhost's resolvability
  assert:
    that: "{{ ipv6_localhost is ansible.utils.resolvable }}"

# TASK [Assert ipv4_localhost's resolvability] *******************************************
# ok: [localhost] => {
#     "changed": false,
#     "msg": "All assertions passed"
# }

# TASK [Assert ipv6_localhost's resolvability] *******************************************
# ok: [localhost] => {
#     "changed": false,
#     "msg": "All assertions passed"
# }

"""

RETURN = """
  data:
    description:
      - If jinja test satisfies plugin expression C(true)
      - If jinja test does not satisfy plugin expression C(false)
"""


@_need_ipaddress
def _resolvable(host):
    """ Test if an IP or name can be resolved via /etc/hosts or DNS """

    params = {"host": host}
    _validate_args("resolvable", DOCUMENTATION, params)

    try:
        ipaddress.ip_address(host)
        ip = True
    except Exception:
        ip = False
    if ip:
        try:
            socket.gethostbyaddr(host)
            return True
        except Exception:
            return False
    else:
        try:
            socket.getaddrinfo(host, None)
            return True
        except Exception:
            return False


class TestModule(object):
    """ network jinja tests """

    test_map = {"resolvable": _resolvable}

    def tests(self):
        return self.test_map



#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright 2020 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type


DOCUMENTATION = """
module: validate
author:
- Bradley Thornton (@cidrblock)
- Ganesh Nalawade (@ganeshrn)
short_description: Validate data with provided criteria
description:
- Validate data with provided criteria based on the validation engine.
version_added: 1.0.0
options:
    data:
        type: raw
        description:
        - Data that will be validated against I(criteria). For the type of data refer to the
          documentation of individual validate plugins.
        required: True
    engine:
        type: str
        description:
        - The name of the validate plugin to use. The engine value should follow
          the fully qualified collection name format, that is
          <org-name>.<collection-name>.<validate-plugin-name>.
        default: ansible.utils.jsonschema
    criteria:
        type: raw
        description:
        - The criteria used for validation of I(data). For the type of criteria refer to the
          documentation of individual validate plugins.
        required: True
notes:
- For the type of options I(data) and I(criteria) refer to the individual validate plugin
  documentation that is represented in the value of I(engine) option.
- For additional plugin configuration options refer to the individual validate plugin
  documentation that is represented by the value of I(engine) option.
- The plugin configuration option can be either passed as task or environment variables.
- The precedence of the validate plugin configurable option is task variables followed
  by the environment variables.
"""

EXAMPLES = r"""
- name: set facts for data and criteria
  ansible.builtin.set_fact:
    data: "{{ lookup('ansible.builtin.file', './validate/data/show_interfaces_iosxr.json')}}"
    criteria: "{{ lookup('ansible.builtin.file', './validate/criteria/jsonschema/show_interfaces_iosxr.json')}}"

- name: validate data in with jsonschema engine (by passing task vars as configurable plugin options)
  ansible.utils.validate:
    data: "{{ data }}"
    criteria: "{{ criteria }}"
    engine: ansible.utils.jsonschema
  vars:
    ansible_jsonschema_draft: draft7
"""

RETURN = r"""
msg:
  description:
  - The msg indicates if the I(data) is valid as per the I(criteria).
  - In case data is valid return success message B(all checks passed).
  - In case data is invalid return error message B(Validation errors were found)
    along with more information on error is available.
  returned: always
  type: str
errors:
  description: The list of errors in I(data) based on the I(criteria).
  returned: when I(data) value is invalid
  type: list
  elements: str
"""



# (c) 2020 Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

import json

from ansible_collections.ansible.utils.tests.unit.compat import unittest
from ansible_collections.ansible.utils.plugins.sub_plugins.cli_parser.json_parser import (
    CliParser,
)


class TestJsonParser(unittest.TestCase):
    def test_json_parser(self):
        test_value = {
            "string": "This is a string",
            "list": ["This", "is", "a", "list"],
            "bool": True,
            "int": 27,
            "dict": {
                "This": "string",
                "is": ["l", "i", "s", "t"],
                "a": True,
                "dict": 42,
            },
        }
        task_args = {"text": json.dumps(test_value)}
        parser = CliParser(task_args=task_args, task_vars=[], debug=False)

        result = parser.parse()
        self.assertEqual(result, {"parsed": test_value})

    def test_invalid_json(self):
        task_args = {"text": "Definitely not JSON"}
        parser = CliParser(task_args=task_args, task_vars=[], debug=False)

        result = parser.parse()
        # Errors are different between Python 2 and 3, so we have to be a bit roundabout.
        self.assertEqual(len(result), 1)
        assert "errors" in result
        self.assertEqual(len(result["errors"]), 1)



# -*- coding: utf-8 -*-
# Copyright 2020 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)


from __future__ import absolute_import, division, print_function

__metaclass__ = type

import unittest
from ansible_collections.ansible.utils.plugins.module_utils.common.argspec_validate import (
    AnsibleArgSpecValidator,
)
from .fixtures.docstring import DOCUMENTATION


class TestSortList(unittest.TestCase):
    def test_simple_pass(self):
        data = {"param_str": "string"}
        aav = AnsibleArgSpecValidator(
            data=data,
            schema=DOCUMENTATION,
            schema_format="doc",
            schema_conditionals={},
            name="test_action",
        )
        valid, errors, _updated_data = aav.validate()
        self.assertTrue(valid)
        self.assertEqual(errors, None)

    def test_simple_defaults(self):
        data = {"param_str": "string"}
        aav = AnsibleArgSpecValidator(
            data=data,
            schema=DOCUMENTATION,
            schema_format="doc",
            schema_conditionals={},
            name="test_action",
        )
        expected = {
            "param_str": "string",
            "param_default": True,
            "params_bool": None,
            "params_dict": None,
        }
        valid, errors, updated_data = aav.validate()
        self.assertTrue(valid)
        self.assertEqual(errors, None)
        self.assertEqual(expected, updated_data)

    def test_simple_fail(self):
        data = {}
        aav = AnsibleArgSpecValidator(
            data=data,
            schema=DOCUMENTATION,
            schema_format="doc",
            schema_conditionals={},
            name="test_action",
        )
        valid, errors, _updated_data = aav.validate()
        self.assertFalse(valid)
        self.assertIn("missing required arguments: param_str", errors)

    def test_simple_fail_no_name(self):
        data = {}
        aav = AnsibleArgSpecValidator(
            data=data,
            schema=DOCUMENTATION,
            schema_format="doc",
            schema_conditionals={},
        )
        valid, errors, _updated_data = aav.validate()
        self.assertFalse(valid)
        self.assertIn("missing required arguments: param_str", errors)

    def test_not_doc(self):
        data = {"param_str": "string"}
        aav = AnsibleArgSpecValidator(
            data=data,
            schema={"argument_spec": {"param_str": {"type": "str"}}},
            schema_format="argspec",
            name="test_action",
        )
        valid, errors, _updated_data = aav.validate()
        self.assertTrue(valid)
        self.assertEqual(errors, None)

    def test_schema_conditional(self):
        data = {"param_str": "string"}
        aav = AnsibleArgSpecValidator(
            data=data,
            schema=DOCUMENTATION,
            schema_format="doc",
            schema_conditionals={
                "required_together": [["param_str", "param_bool"]]
            },
            name="test_action",
        )
        valid, errors, _updated_data = aav.validate()
        self.assertFalse(valid)
        self.assertIn(
            "parameters are required together: param_str, param_bool", errors
        )

    def test_unsupported_param(self):
        data = {"param_str": "string", "not_valid": "string"}
        aav = AnsibleArgSpecValidator(
            data=data,
            schema=DOCUMENTATION,
            schema_format="doc",
            name="test_action",
            # other_args={'bypass_checks': True},
        )
        valid, errors, _updated_data = aav.validate()
        self.assertFalse(valid)
        self.assertIn(
            "Unsupported parameters for 'test_action' module: not_valid",
            errors,
        )

    def test_other_args(self):
        data = {"param_str": "string"}
        aav = AnsibleArgSpecValidator(
            data=data,
            schema=DOCUMENTATION,
            schema_format="doc",
            name="test_action",
            other_args={"bypass_checks": True},
        )
        valid, errors, _updated_data = aav.validate()
        self.assertTrue(valid)
        self.assertIsNone(errors)

    def test_invalid_spec(self):
        data = {}
        aav = AnsibleArgSpecValidator(
            data=data,
            schema={"not_valid": True},
            schema_format="argspec",
            name="test_action",
            other_args={"bypass_checks": True},
        )
        valid, errors, _updated_data = aav.validate()
        self.assertFalse(valid)
        self.assertIn("Invalid keys found: not_valid", errors)



# -*- coding: utf-8 -*-
# Copyright 2020 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)


"""
flatten a complex object to dot bracket notation
"""
from __future__ import absolute_import, division, print_function

__metaclass__ = type

DOCUMENTATION = """
    name: get_path
    author: Bradley Thornton (@cidrblock)
    version_added: "1.0.0"
    short_description: Retrieve the value in a variable using a path
    description:
        - Use a I(path) to retrieve a nested value from a I(var).
        - B(get_path) is also available as a B(lookup plugin) for convenience.
        - Using the parameters below- C(var|ansible.utils.get_path(path, wantlist))
    options:
      var:
        description:
        - The variable from which the value should be extracted.
        - This option represents the value that is passed to the filter plugin in pipe format.
        - For example C(config_data|ansible.utils.get_path()), in this case C(config_data) represents this option.
        type: raw
        required: True
      path:
        description:
        - The I(path) in the I(var) to retrieve the value of.
        - The I(path) needs to be a valid jinja path.
        type: str
        required: True
      wantlist:
        description:
        - If set to C(True), the return value will always be a list.
        type: bool

    notes:
"""

EXAMPLES = r"""
- ansible.builtin.set_fact:
    a:
      b:
        c:
          d:
          - 0
          - 1
          e:
          - True
          - False

- name: Retrieve a value deep inside a using a path
  ansible.builtin.set_fact:
    value: "{{ a|ansible.utils.get_path(path) }}"
  vars:
    path: b.c.d[0]

# TASK [Retrieve a value deep inside a using a path] ******************
# ok: [localhost] => changed=false
#   ansible_facts:
#     value: '0'


#### Working with hostvars

- name: Retrieve a value deep inside all of the host's vars
  ansible.builtin.set_fact:
    value: "{{ look_in|ansible.utils.get_path(look_for) }}"
  vars:
    look_in: "{{ hostvars[inventory_hostname] }}"
    look_for: a.b.c.d[0]

# TASK [Retrieve a value deep inside all of the host's vars] ********
# ok: [nxos101] => changed=false
#   ansible_facts:
#     as_filter: '0'
#     as_lookup: '0'


#### Used alongside ansible.utils.to_paths

- name: Get the paths for the object
  ansible.builtin.set_fact:
    paths: "{{ a|ansible.utils.to_paths(prepend='a') }}"

- name: Retrieve the value of each path from vars
  ansible.builtin.debug:
    msg: "The value of path {{ path }} in vars is {{ value }}"
  loop: "{{ paths.keys()|list }}"
  loop_control:
    label: "{{ item }}"
  vars:
    path: "{{ item }}"
    value: "{{ vars|ansible.utils.get_path(item) }}"

# TASK [Get the paths for the object] *******************************
# ok: [nxos101] => changed=false
#   ansible_facts:
#     paths:
#       a.b.c.d[0]: 0
#       a.b.c.d[1]: 1
#       a.b.c.e[0]: True
#       a.b.c.e[1]: False

# TASK [Retrieve the value of each path from vars] ******************
# ok: [nxos101] => (item=a.b.c.d[0]) =>
#   msg: The value of path a.b.c.d[0] in vars is 0
# ok: [nxos101] => (item=a.b.c.d[1]) =>
#   msg: The value of path a.b.c.d[1] in vars is 1
# ok: [nxos101] => (item=a.b.c.e[0]) =>
#   msg: The value of path a.b.c.e[0] in vars is True
# ok: [nxos101] => (item=a.b.c.e[1]) =>
#   msg: The value of path a.b.c.e[1] in vars is False


#### Working with complex structures and transforming results

- name: Retrieve the current interface config
  cisco.nxos.nxos_interfaces:
    state: gathered
  register: interfaces

- name: Get the description of several interfaces
  ansible.builtin.debug:
    msg: "{{ rekeyed|ansible.utils.get_path(item) }}"
  vars:
    rekeyed:
      by_name: "{{ interfaces.gathered|ansible.builtin.rekey_on_member('name') }}"
  loop:
  - by_name['Ethernet1/1'].description
  - by_name['Ethernet1/2'].description|upper
  - by_name['Ethernet1/3'].description|default('')


# TASK [Get the description of several interfaces] ******************
# ok: [nxos101] => (item=by_name['Ethernet1/1'].description) => changed=false
#   msg: Configured by ansible
# ok: [nxos101] => (item=by_name['Ethernet1/2'].description|upper) => changed=false
#   msg: CONFIGURED BY ANSIBLE
# ok: [nxos101] => (item=by_name['Ethernet1/3'].description|default('')) => changed=false
#   msg: ''

"""

from ansible.errors import AnsibleFilterError
from jinja2.filters import environmentfilter

from ansible_collections.ansible.utils.plugins.module_utils.common.get_path import (
    get_path,
)

from ansible_collections.ansible.utils.plugins.module_utils.common.argspec_validate import (
    AnsibleArgSpecValidator,
)


@environmentfilter
def _get_path(*args, **kwargs):
    """Retrieve the value in a variable using a path."""
    keys = ["environment", "var", "path"]
    data = dict(zip(keys, args))
    data.update(kwargs)
    environment = data.pop("environment")
    aav = AnsibleArgSpecValidator(
        data=data, schema=DOCUMENTATION, name="get_path"
    )
    valid, errors, updated_data = aav.validate()
    if not valid:
        raise AnsibleFilterError(errors)
    updated_data["environment"] = environment
    return get_path(**updated_data)


class FilterModule(object):
    """ path filters """

    def filters(self):
        return {"get_path": _get_path}



# -*- coding: utf-8 -*-
# Copyright 2021 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

"""
Test plugin file for netaddr tests: ipv6_ipv4_mapped
"""

from __future__ import absolute_import, division, print_function
from ansible_collections.ansible.utils.plugins.plugin_utils.base.ipaddress_utils import (
    ip_address,
    _need_ipaddress,
    _validate_args,
)

__metaclass__ = type

DOCUMENTATION = """
    name: ipv6_ipv4_mapped
    author: Priyam Sahoo (@priyamsahoo)
    version_added: "2.2.0"
    short_description: Test if something appears to be a mapped IPv6 to IPv4 mapped address
    description:
        - This plugin checks if the provided value is a valid IPv4-mapped IPv6 address
    options:
        ip:
            description:
            - A string that represents the value against which the test is going to be performed
            - For example:
                - "::FFFF:10.1.1.1"
                - "::AAAA:10.1.1.1"
                - "helloworld"
            type: str
            required: True
    notes:
"""

EXAMPLES = r"""

#### Simple examples

- name: Check if ::FFFF:10.1.1.1 is a valid IPv4-mapped IPv6 address
  ansible.builtin.set_fact:
    data: "{{ '::FFFF:10.1.1.1' is ansible.utils.ipv6_ipv4_mapped }}"

# TASK [Check if ::FFFF:10.1.1.1 is a valid IPv4-mapped IPv6 address] *************
# ok: [localhost] => {
#     "ansible_facts": {
#         "data": true
#     },
#     "changed": false
# }

- name: Check if ::AAAA:10.1.1.1 is not a valid IPv4-mapped IPv6 address
  ansible.builtin.set_fact:
    data: "{{ '::AAAA:10.1.1.1' is not ansible.utils.ipv6_ipv4_mapped }}"

# TASK [Check if ::AAAA:10.1.1.1 is not a valid IPv4-mapped IPv6 address] ******************
# ok: [localhost] => {
#     "ansible_facts": {
#         "data": true
#     },
#     "changed": false
# }

- name: Check if helloworld is not a valid IPv4-mapped IPv6 address
  ansible.builtin.set_fact:
    data: "{{ 'helloworld' is not ansible.utils.ipv6_ipv4_mapped }}"

# TASK [Check if helloworld is not a valid IPv4-mapped IPv6 address] ***********************
# ok: [localhost] => {
#     "ansible_facts": {
#         "data": true
#     },
#     "changed": false
# }

"""

RETURN = """
  data:
    description:
      - If jinja test satisfies plugin expression C(true)
      - If jinja test does not satisfy plugin expression C(false)
"""


@_need_ipaddress
def _ipv6_ipv4_mapped(ip):
    """ Test if something appears to be a mapped IPv6 to IPv4 mapped address """

    params = {"ip": ip}
    _validate_args("ipv6_ipv4_mapped", DOCUMENTATION, params)

    try:
        if ip_address(ip).ipv4_mapped is None:
            return False
        return True
    except Exception:
        return False


class TestModule(object):
    """ network jinja test"""

    test_map = {"ipv6_ipv4_mapped": _ipv6_ipv4_mapped}

    def tests(self):
        return self.test_map



# -*- coding: utf-8 -*-
# Copyright 2020 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

import re
import unittest
from ansible.playbook.task import Task
from ansible.template import Templar

from ansible_collections.ansible.utils.plugins.action.fact_diff import (
    ActionModule,
)

try:
    from unittest.mock import MagicMock  # pylint:disable=syntax-error
except ImportError:
    from mock import MagicMock


class TestUpdate_Fact(unittest.TestCase):
    def setUp(self):
        task = MagicMock(Task)
        play_context = MagicMock()
        play_context.check_mode = False
        connection = MagicMock()
        fake_loader = {}
        templar = Templar(loader=fake_loader)
        self._plugin = ActionModule(
            task=task,
            connection=connection,
            play_context=play_context,
            loader=fake_loader,
            templar=templar,
            shared_loader_obj=None,
        )
        self._plugin._task.action = "fact_diff"
        self._task_vars = {"inventory_hostname": "mockdevice"}

    def test_argspec_no_updates(self):
        """Check passing invalid argspec"""
        self._plugin._task.args = {"before": True}
        result = self._plugin.run(task_vars=self._task_vars)
        self.assertTrue(result["failed"])
        self.assertIn("missing required arguments: after", result["msg"])

    def test_same(self):
        """Ensure two equal string don't create a diff"""
        before = "Lorem ipsum dolor sit amet"
        after = before
        self._plugin._task.args = {"before": before, "after": after}
        result = self._plugin.run(task_vars=self._task_vars)
        self.assertFalse(result["changed"])
        self.assertEqual([], result["diff_lines"])
        self.assertEqual("", result["diff_text"])

    def test_string(self):
        """Compare two strings"""
        before = "Lorem ipsum dolor sit amet, consectetur adipiscing elit"
        after = "Lorem ipsum dolor sit amet, AAA consectetur adipiscing elit"
        self._plugin._task.args = {"before": before, "after": after}
        result = self._plugin.run(task_vars=self._task_vars)
        self.assertTrue(result["changed"])
        self.assertIn("-" + before, result["diff_lines"])
        self.assertIn("-" + before, result["diff_text"])
        self.assertIn("+" + after, result["diff_lines"])
        self.assertIn("+" + after, result["diff_text"])

    def test_string_skip_lines(self):
        """Compare two string, with skip_lines"""
        before = "Lorem ipsum dolor sit amet, consectetur adipiscing elit"
        after = "Lorem ipsum dolor sit amet, AAA consectetur adipiscing elit"
        self._plugin._task.args = {
            "before": before,
            "after": after,
            "plugin": {"vars": {"skip_lines": "^Lorem"}},
        }
        result = self._plugin.run(task_vars=self._task_vars)
        self.assertFalse(result["changed"])
        self.assertEqual([], result["diff_lines"])
        self.assertEqual("", result["diff_text"])

    def test_same_list(self):
        """Compare two lists that are the same"""
        before = [0, 1, 2, 3]
        after = before
        self._plugin._task.args = {"before": before, "after": after}
        result = self._plugin.run(task_vars=self._task_vars)
        self.assertFalse(result["changed"])
        self.assertEqual([], result["diff_lines"])
        self.assertEqual("", result["diff_text"])

    def test_diff_list_skip_lines(self):
        """Compare two lists, with skip_lines"""
        before = [0, 1, 2]
        after = [0, 1, 2, 3]
        self._plugin._task.args = {
            "before": before,
            "after": after,
            "plugin": {"vars": {"skip_lines": "3"}},
        }
        result = self._plugin.run(task_vars=self._task_vars)
        self.assertFalse(result["changed"])
        self.assertEqual([], result["diff_lines"])
        self.assertEqual("", result["diff_text"])

    def test_diff_list(self):
        """Compare two lists with differences"""
        before = [0, 1, 2, 3]
        after = [0, 1, 2, 4]
        self._plugin._task.args = {"before": before, "after": after}
        result = self._plugin.run(task_vars=self._task_vars)
        self.assertTrue(result["changed"])
        self.assertIn("-3", result["diff_lines"])
        self.assertIn("-3", result["diff_text"])
        self.assertIn("+4", result["diff_lines"])
        self.assertIn("+4", result["diff_text"])

    def test_same_dict(self):
        """Compare two dicts that are the same"""
        before = {"a": {"b": {"c": {"d": [0, 1, 2]}}}}
        after = before
        self._plugin._task.args = {"before": before, "after": after}
        result = self._plugin.run(task_vars=self._task_vars)
        self.assertFalse(result["changed"])
        self.assertEqual([], result["diff_lines"])
        self.assertEqual("", result["diff_text"])

    def test_diff_dict_skip_lines(self):
        """Compare two dicts, with skip_lines"""
        before = {"a": {"b": {"c": {"d": [0, 1, 2]}}}}
        after = {"a": {"b": {"c": {"d": [0, 1, 2, 3]}}}}
        self._plugin._task.args = {
            "before": before,
            "after": after,
            "plugin": {"vars": {"skip_lines": "3"}},
        }
        result = self._plugin.run(task_vars=self._task_vars)
        self.assertFalse(result["changed"])
        self.assertEqual([], result["diff_lines"])
        self.assertEqual("", result["diff_text"])

    def test_diff_dict(self):
        """Compare two dicts that are different"""
        before = {"a": {"b": {"c": {"d": [0, 1, 2, 3]}}}}
        after = {"a": {"b": {"c": {"d": [0, 1, 2, 4]}}}}
        self._plugin._task.args = {"before": before, "after": after}
        result = self._plugin.run(task_vars=self._task_vars)
        self.assertTrue(result["changed"])
        mlines = [
            line for line in result["diff_lines"] if re.match(r"^-\s+3$", line)
        ]
        self.assertEqual(1, len(mlines))
        mlines = [
            line
            for line in result["diff_lines"]
            if re.match(r"^\+\s+4$", line)
        ]
        self.assertEqual(1, len(mlines))

    def test_invalid_diff_engine_not_collection(self):
        """Check passing invalid argspec"""
        self._plugin._task.args = {
            "before": True,
            "after": True,
            "plugin": {"name": "a"},
        }
        result = self._plugin.run(task_vars=self._task_vars)
        self.assertTrue(result["failed"])
        self.assertIn(
            "Plugin name should be provided as a full name including collection",
            result["msg"],
        )

    def test_invalid_diff_engine_not_valid(self):
        """Check passing invalid argspec"""
        self._plugin._task.args = {
            "before": True,
            "after": True,
            "plugin": {"name": "a.b.c"},
        }
        result = self._plugin.run(task_vars=self._task_vars)
        self.assertTrue(result["failed"])
        self.assertIn("Error loading plugin 'a.b.c'", result["msg"])

    def test_invalid_regex(self):
        """Check with invalid regex"""
        before = True
        after = False
        self._plugin._task.args = {
            "before": before,
            "after": after,
            "plugin": {"vars": {"skip_lines": "+"}},
        }
        result = self._plugin.run(task_vars=self._task_vars)
        self.assertTrue(result["failed"])
        self.assertIn("The regex '+', is not valid", result["msg"])

    def test_fail_plugin(self):
        """Simulate a diff plugin failure"""
        self._plugin._result = {}
        result = self._plugin._run_diff(None)
        self.assertIsNone(result)
        self.assertTrue(self._plugin._result["failed"])
        self.assertIn(
            "'NoneType' object has no attribute 'diff'",
            self._plugin._result["msg"],
        )



# -*- coding: utf-8 -*-
# Copyright 2020 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)


from __future__ import absolute_import, division, print_function

__metaclass__ = type

import unittest
from ansible_collections.ansible.utils.plugins.module_utils.common.get_path import (
    get_path,
)


from ansible.template import Templar


class TestGetPath(unittest.TestCase):
    def setUp(self):
        self._environment = Templar(loader=None).environment

    def test_get_path_pass(self):
        var = {"a": {"b": {"c": {"d": [0, 1]}}}}
        path = "a.b.c.d[0]"
        result = get_path(
            var, path, environment=self._environment, wantlist=False
        )
        expected = "0"
        self.assertEqual(result, expected)

    def test_get_path_pass_wantlist(self):
        var = {"a": {"b": {"c": {"d": [0, 1]}}}}
        path = "a.b.c.d[0]"
        result = get_path(
            var, path, environment=self._environment, wantlist=True
        )
        expected = ["0"]
        self.assertEqual(result, expected)

    def test_get_path_fail(self):
        var = {"a": {"b": {"c": {"d": [0, 1]}}}}
        path = "a.b.e"
        expected = "dict object' has no attribute 'e'"
        with self.assertRaises(Exception) as exc:
            get_path(var, path, environment=self._environment, wantlist=False)
        self.assertIn(expected, str(exc.exception))



# -*- coding: utf-8 -*-
# Copyright 2021 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

"""
Unit test file for netaddr test plugin: ipv6_ipv4_mapped
"""

from __future__ import absolute_import, division, print_function

__metaclass__ = type

import unittest
from ansible_collections.ansible.utils.plugins.test.ipv6_ipv4_mapped import (
    _ipv6_ipv4_mapped,
)


class TestIpV6IpV4Mapped(unittest.TestCase):
    def setUp(self):
        pass

    def test_invalid_data(self):
        """Check passing invalid argspec"""

        # missing argument
        with self.assertRaises(TypeError) as error:
            _ipv6_ipv4_mapped()
        self.assertIn("argument", str(error.exception))

    def test_valid_data(self):
        """Check passing valid data as per criteria"""

        result = _ipv6_ipv4_mapped(ip="::FFFF:10.1.1.1")
        self.assertEqual(result, True)

        result = _ipv6_ipv4_mapped(ip="::AAAA:10.1.1.1")
        self.assertEqual(result, False)

        result = _ipv6_ipv4_mapped(ip="string")
        self.assertEqual(result, False)



# -*- coding: utf-8 -*-
# Copyright 2021 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

"""
Unit test file for netaddr test plugin: subnet_of
"""

from __future__ import absolute_import, division, print_function

__metaclass__ = type

import unittest
from ansible_collections.ansible.utils.plugins.test.subnet_of import _subnet_of


class TestSubnetOf(unittest.TestCase):
    def setUp(self):
        pass

    def test_invalid_data(self):
        """Check passing invalid argspec"""

        # missing argument
        with self.assertRaises(TypeError) as error:
            _subnet_of()
        self.assertIn("argument", str(error.exception))

        with self.assertRaises(TypeError) as error:
            _subnet_of(network_a="10.1.1.0/24")
        self.assertIn("argument", str(error.exception))

        with self.assertRaises(TypeError) as error:
            _subnet_of(network_b="10.0.0.0/8")
        self.assertIn("argument", str(error.exception))

    def test_valid_data(self):
        """Check passing valid data as per criteria"""

        result = _subnet_of(network_a="10.1.1.0/24", network_b="10.0.0.0/8")
        self.assertEqual(result, True)

        result = _subnet_of(network_a="10.0.0.0/8", network_b="10.1.1.0/24")
        self.assertEqual(result, False)

        result = _subnet_of(network_a="192.168.1.0/24", network_b="10.0.0.0/8")
        self.assertEqual(result, False)



# -*- coding: utf-8 -*-
# Copyright 2020 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

import copy
import unittest
from jinja2 import Template, TemplateSyntaxError
from ansible.playbook.task import Task
from ansible.template import Templar

from ansible_collections.ansible.utils.plugins.action.update_fact import (
    ActionModule,
)

try:
    from unittest.mock import MagicMock  # pylint:disable=syntax-error
except ImportError:
    from mock import MagicMock


VALID_DATA = {
    "a": {
        "b": {"4.4": [{"1": {5: {"foo": 123}}}], 5.5: "float5.5"},
        "127.0.0.1": "localhost",
    }
}

VALID_TESTS = [
    {
        "path": 'a.b["4.4"][0]["1"].5[\'foo\']',
        "split": ["a", "b", "4.4", 0, "1", 5, "foo"],
        "template_result": "123",
    },
    {
        "path": 'a.b["4.4"][0]["1"].5[\'foo\']',
        "split": ["a", "b", "4.4", 0, "1", 5, "foo"],
        "template_result": "123",
    },
    {
        "path": "a.b[5.5]",
        "split": ["a", "b", 5.5],
        "template_result": "float5.5",
    },
    {
        "path": "a['127.0.0.1']",
        "split": ["a", "127.0.0.1"],
        "template_result": "localhost",
    },
    {
        "path": "a.b['4.4'].0['1'].5['foo']",
        "split": ["a", "b", "4.4", 0, "1", 5, "foo"],
        "template_result": "123",
    },
]


INVALID_JINJA = [
    {
        "path": "a.'1'",
        "note": "quoted values are required to be in brackets",
        "error": "expected name or number",
    },
    {
        "path": "a.[1]",
        "note": "brackets can't follow dots",
        "error": "expected name or number",
    },
    {
        "path": 'a.b["4.4"][0]["1"]."5"[\'foo\']',
        "note": "quoted values are required to be in brackets",
        "error": "expected name or number",
    },
]


class TestUpdate_Fact(unittest.TestCase):
    def setUp(self):
        task = MagicMock(Task)
        play_context = MagicMock()
        play_context.check_mode = False
        connection = MagicMock()
        fake_loader = {}
        templar = Templar(loader=fake_loader)
        self._plugin = ActionModule(
            task=task,
            connection=connection,
            play_context=play_context,
            loader=fake_loader,
            templar=templar,
            shared_loader_obj=None,
        )
        self._plugin._task.action = "update_fact"

    def test_argspec_no_updates(self):
        """Check passing invalid argspec"""
        self._plugin._task.args = {"a": 10}
        with self.assertRaises(Exception) as error:
            self._plugin.run(task_vars=None)
        self.assertIn(
            "missing required arguments: updates", str(error.exception)
        )

    def test_argspec_none(self):
        """Check passing a dict"""
        self._plugin._task.args = {}
        with self.assertRaises(Exception) as error:
            self._plugin.run(task_vars=None)
        self.assertIn(
            "missing required arguments: updates", str(error.exception)
        )

    def test_valid_jinja(self):
        for test in VALID_TESTS:
            tmplt = Template("{{" + test["path"] + "}}")
            result = tmplt.render(VALID_DATA)
            self.assertEqual(result, test["template_result"])

    def test_invalid_jinja(self):
        for test in INVALID_JINJA:
            with self.assertRaises(TemplateSyntaxError) as error:
                Template("{{" + test["path"] + "}}")
            self.assertIn(test["error"], str(error.exception))

    def test_fields(self):
        """Check the parsing of a path into it's parts"""
        for stest in VALID_TESTS:
            result = self._plugin._field_split(stest["path"])
            self.assertEqual(result, stest["split"])

    def test_missing_var(self):
        """Check for a missing fact"""
        self._plugin._task.args = {"updates": [{"path": "a.b.c", "value": 5}]}
        with self.assertRaises(Exception) as error:
            self._plugin.run(task_vars={"vars": {}})
        self.assertIn(
            "'a' was not found in the current facts.", str(error.exception)
        )

    def test_run_simple(self):
        """Confirm a valid argspec passes"""
        task_vars = {"vars": {"a": {"b": [1, 2, 3]}}}
        expected = copy.deepcopy(task_vars["vars"])
        expected["a"]["b"] = 5
        expected.update({"changed": True})
        self._plugin._task.args = {"updates": [{"path": "a.b", "value": 5}]}
        result = self._plugin.run(task_vars=task_vars)
        self.assertEqual(result, expected)

    def test_run_multiple(self):
        """Confirm multiple paths passes"""
        task_vars = {
            "vars": {"a": {"b1": [1, 2, 3], "b2": {"c": "123", "d": False}}}
        }
        expected = {"a": {"b1": [1, 2, 3, 4], "b2": {"c": 456, "d": True}}}
        expected.update({"changed": True})
        self._plugin._task.args = {
            "updates": [
                {"path": "a.b1.3", "value": 4},
                {"path": "a.b2.c", "value": 456},
                {"path": "a.b2.d", "value": True},
            ]
        }
        result = self._plugin.run(task_vars=task_vars)
        self.assertEqual(result, expected)

    def test_run_replace_in_list(self):
        """Replace in list"""
        task_vars = {"vars": {"a": {"b": [1, 2, 3]}}}
        expected = copy.deepcopy(task_vars["vars"])
        expected["a"]["b"][1] = 5
        expected.update({"changed": True})
        self._plugin._task.args = {"updates": [{"path": "a.b.1", "value": 5}]}
        result = self._plugin.run(task_vars=task_vars)
        self.assertEqual(result, expected)

    def test_run_append_to_list(self):
        """Append to list"""
        task_vars = {"vars": {"a": {"b": [1, 2, 3]}}}
        expected = copy.deepcopy(task_vars["vars"])
        expected["a"]["b"].append(4)
        expected.update({"changed": True})
        self._plugin._task.args = {"updates": [{"path": "a.b.3", "value": 4}]}
        result = self._plugin.run(task_vars=task_vars)
        self.assertEqual(result, expected)

    def test_run_bracket_single_quote(self):
        """Bracket notation sigle quote"""
        task_vars = {"vars": {"a": {"b": [1, 2, 3]}}}
        expected = copy.deepcopy(task_vars["vars"])
        expected["a"]["b"].append(4)
        expected.update({"changed": True})
        self._plugin._task.args = {
            "updates": [{"path": "a['b'][3]", "value": 4}]
        }
        result = self._plugin.run(task_vars=task_vars)
        self.assertEqual(result, expected)

    def test_run_bracket_double_quote(self):
        """Bracket notation double quote"""
        task_vars = {"vars": {"a": {"b": [1, 2, 3]}}}
        expected = copy.deepcopy(task_vars["vars"])
        expected["a"]["b"].append(4)
        expected.update({"changed": True})
        self._plugin._task.args = {
            "updates": [{"path": 'a["b"][3]', "value": 4}]
        }
        result = self._plugin.run(task_vars=task_vars)
        self.assertEqual(result, expected)

    def test_run_int_dict_keys(self):
        """Integer dict keys"""
        task_vars = {"vars": {"a": {0: [1, 2, 3]}}}
        expected = copy.deepcopy(task_vars["vars"])
        expected["a"][0][0] = 0
        expected.update({"changed": True})
        self._plugin._task.args = {"updates": [{"path": "a.0.0", "value": 0}]}
        result = self._plugin.run(task_vars=task_vars)
        self.assertEqual(result, expected)

    def test_run_int_as_string(self):
        """Integer dict keys as string"""
        task_vars = {"vars": {"a": {"0": [1, 2, 3]}}}
        expected = copy.deepcopy(task_vars["vars"])
        expected["a"]["0"][0] = 0
        expected.update({"changed": True})
        self._plugin._task.args = {
            "updates": [{"path": 'a["0"].0', "value": 0}]
        }
        result = self._plugin.run(task_vars=task_vars)
        self.assertEqual(result, expected)

    def test_run_invalid_path_quote_after_dot(self):
        """Invalid path format"""
        self._plugin._task.args = {"updates": [{"path": "a.'b'", "value": 0}]}
        with self.assertRaises(Exception) as error:
            self._plugin.run(task_vars={"vars": {}})
        self.assertIn("malformed", str(error.exception))

    def test_run_invalid_path_bracket_after_dot(self):
        """Invalid path format"""
        self._plugin._task.args = {
            "updates": [{"path": "a.['b']", "value": 0}]
        }
        with self.assertRaises(Exception) as error:
            self._plugin.run(task_vars={"vars": {}})
        self.assertIn("malformed", str(error.exception))

    def test_run_invalid_key_start_with_dot(self):
        """Invalid key format"""
        self._plugin._task.args = {"updates": [{"path": ".abc", "value": 0}]}
        with self.assertRaises(Exception) as error:
            self._plugin.run(task_vars={"vars": {}})
        self.assertIn("malformed", str(error.exception))

    def test_run_no_update_list(self):
        """Confirm no change when same"""
        task_vars = {"vars": {"a": {"b": [1, 2, 3]}}}
        expected = copy.deepcopy(task_vars["vars"])
        expected["a"]["b"] = [1, 2, 3]
        expected.update({"changed": False})
        self._plugin._task.args = {"updates": [{"path": "a.b.0", "value": 1}]}
        result = self._plugin.run(task_vars=task_vars)
        self.assertEqual(result, expected)

    def test_run_no_update_dict(self):
        """Confirm no change when same"""
        task_vars = {"vars": {"a": {"b": [1, 2, 3]}}}
        expected = copy.deepcopy(task_vars["vars"])
        expected["a"]["b"] = [1, 2, 3]
        expected.update({"changed": False})
        self._plugin._task.args = {
            "updates": [{"path": "a.b", "value": [1, 2, 3]}]
        }
        result = self._plugin.run(task_vars=task_vars)
        self.assertEqual(result, expected)

    def test_run_missing_key(self):
        """Confirm error when key not found"""
        task_vars = {"vars": {"a": {"b": 1}}}
        self._plugin._task.args = {"updates": [{"path": "a.c.d", "value": 1}]}
        with self.assertRaises(Exception) as error:
            self._plugin.run(task_vars=task_vars)
        self.assertIn("the key 'c' was not found", str(error.exception))

    def test_run_list_not_int(self):
        """Confirm error when key not found"""
        task_vars = {"vars": {"a": {"b": [1]}}}
        self._plugin._task.args = {
            "updates": [{"path": "a.b['0']", "value": 2}]
        }
        with self.assertRaises(Exception) as error:
            self._plugin.run(task_vars=task_vars)
        self.assertIn(
            "index provided was not an integer", str(error.exception)
        )

    def test_run_list_not_long(self):
        """List not long enough"""
        task_vars = {"vars": {"a": {"b": [0]}}}
        self._plugin._task.args = {"updates": [{"path": "a.b.2", "value": 2}]}
        with self.assertRaises(Exception) as error:
            self._plugin.run(task_vars=task_vars)
        self.assertIn(
            "not long enough for item #2 to be set", str(error.exception)
        )

    def test_not_mutable_sequence_or_mapping(self):
        """Confirm graceful fail when immutable object
        This should never happen in the real world
        """
        obj = {"a": frozenset([1, 2, 3])}
        path = ["a", 0]
        val = 9
        with self.assertRaises(Exception) as error:
            self._plugin.set_value(obj, path, val)
        self.assertIn("can only modify mutable objects", str(error.exception))

    def test_run_not_dotted_success_one(self):
        """Test with a not dotted key"""
        task_vars = {"vars": {"a": 0}}
        expected = copy.deepcopy(task_vars["vars"])
        expected["a"] = 1
        expected.update({"changed": True})
        self._plugin._task.args = {"updates": [{"path": "a", "value": 1}]}
        result = self._plugin.run(task_vars=task_vars)
        self.assertEqual(result, expected)

    def test_run_not_dotted_success_three(self):
        """Test with a not dotted key longer"""
        task_vars = {"vars": {"abc": 0}}
        expected = copy.deepcopy(task_vars["vars"])
        expected["abc"] = 1
        expected.update({"changed": True})
        self._plugin._task.args = {"updates": [{"path": "abc", "value": 1}]}
        result = self._plugin.run(task_vars=task_vars)
        self.assertEqual(result, expected)

    def test_run_not_dotted_fail_missing(self):
        """Test with a not dotted key, missing"""
        task_vars = {"vars": {"abc": 0}}
        self._plugin._task.args = {"updates": [{"path": "123", "value": 1}]}
        with self.assertRaises(Exception) as error:
            self._plugin.run(task_vars=task_vars)
        self.assertIn(
            "'123' was not found in the current facts", str(error.exception)
        )

    def test_run_not_dotted_success_same(self):
        """Test with a not dotted key, no change"""
        task_vars = {"vars": {"a": 0}}
        expected = copy.deepcopy(task_vars["vars"])
        expected.update({"changed": False})
        self._plugin._task.args = {"updates": [{"path": "a", "value": 0}]}
        result = self._plugin.run(task_vars=task_vars)
        self.assertEqual(result, expected)

    def test_run_looks_like_a_bool(self):
        """Test with a key that looks like a bool"""
        task_vars = {"vars": {"a": {"True": 0}}}
        expected = copy.deepcopy(task_vars["vars"])
        expected["a"]["True"] = 1
        expected.update({"changed": True})
        self._plugin._task.args = {
            "updates": [{"path": "a['True']", "value": 1}]
        }
        result = self._plugin.run(task_vars=task_vars)
        self.assertEqual(result, expected)



# -*- coding: utf-8 -*-
# Copyright 2020 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)


from __future__ import absolute_import, division, print_function

__metaclass__ = type

import json
import heapq
import os
import unittest
from ansible_collections.ansible.utils.plugins.module_utils.common.get_path import (
    get_path,
)
from ansible_collections.ansible.utils.plugins.module_utils.common.to_paths import (
    to_paths,
)

from ansible.template import Templar


class TestToPaths(unittest.TestCase):
    def setUp(self):
        self._environment = Templar(loader=None).environment

    def test_to_paths(self):
        var = {"a": {"b": {"c": {"d": [0, 1]}}}}
        expected = {"a.b.c.d[0]": 0, "a.b.c.d[1]": 1}
        result = to_paths(var, prepend=None, wantlist=None)
        self.assertEqual(result, expected)

    def test_to_paths_wantlist(self):
        var = {"a": {"b": {"c": {"d": [0, 1]}}}}
        expected = [{"a.b.c.d[0]": 0, "a.b.c.d[1]": 1}]
        result = to_paths(var, prepend=None, wantlist=True)
        self.assertEqual(result, expected)

    def test_to_paths_special_char(self):
        var = {"a": {"b": {"c": {"Eth1/1": True}}}}
        expected = [{"a.b.c['Eth1/1']": True}]
        result = to_paths(var, prepend=None, wantlist=True)
        self.assertEqual(result, expected)

    def test_to_paths_prepend(self):
        var = {"a": {"b": {"c": {"d": [0, 1]}}}}
        expected = [{"var.a.b.c.d[0]": 0, "var.a.b.c.d[1]": 1}]
        result = to_paths(var, wantlist=True, prepend="var")
        self.assertEqual(result, expected)

    def test_roundtrip_large(self):
        """Test the 1000 longest keys, otherwise this takes a _really_ long time"""
        big_json_path = os.path.join(
            os.path.dirname(__file__), "fixtures", "large.json"
        )
        with open(big_json_path) as fhand:
            big_json = fhand.read()
        var = json.loads(big_json)
        paths = to_paths(var, prepend=None, wantlist=None)
        to_tests = heapq.nlargest(1000, list(paths.keys()), key=len)
        for to_test in to_tests:
            gotten = get_path(
                var, to_test, environment=self._environment, wantlist=False
            )
            self.assertEqual(gotten, paths[to_test])

    def test_to_paths_empty_list(self):
        var = {"a": []}
        expected = {"a": []}
        result = to_paths(var, prepend=None, wantlist=None)
        self.assertEqual(result, expected)

    def test_to_paths_list_of_empty_list(self):
        var = {"a": [[], []]}
        expected = {"a[0]": [], "a[1]": []}
        result = to_paths(var, prepend=None, wantlist=None)
        self.assertEqual(result, expected)

    def test_to_paths_empty_mapping(self):
        var = {"a": {}}
        expected = {"a": {}}
        result = to_paths(var, prepend=None, wantlist=None)
        self.assertEqual(result, expected)

    def test_to_paths_list_of_empty_mapping(self):
        var = [{}, {}]
        expected = {"[0]": {}, "[1]": {}}
        result = to_paths(var, prepend=None, wantlist=None)
        self.assertEqual(result, expected)

    def test_to_paths_only_empty_list(self):
        var = []
        expected = []
        result = to_paths(var, prepend=None, wantlist=None)
        self.assertEqual(result, expected)

    def test_to_paths_only_empty_mapping(self):
        var = {}
        expected = {}
        result = to_paths(var, prepend=None, wantlist=None)
        self.assertEqual(result, expected)



# -*- coding: utf-8 -*-
# Copyright 2020 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)


"""
flatten a complex object to dot bracket notation
"""
from __future__ import absolute_import, division, print_function

__metaclass__ = type


def get_path(var, path, environment, wantlist):
    """Get the value of a path within an object

    :param var: The var from which the value is retrieved
    :type var: should be dict or list, but jinja can sort that out
    :param path: The path to get
    :type path: should be a string but jinja can sort that out
    :param environment: The jinja Environment
    :type environment: Environment
    :return: The result of the jinja evaluation
    :rtype: any
    """
    string_to_variable = "{{ %s }}" % path
    result = environment.from_string(string_to_variable).render(**var)
    if wantlist:
        return [result]
    return result



# -*- coding: utf-8 -*-
# Copyright 2020 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)


from __future__ import absolute_import, division, print_function

__metaclass__ = type

import unittest
from ansible_collections.ansible.utils.plugins.module_utils.common.utils import (
    sort_list,
)


class TestSortList(unittest.TestCase):
    def test_simple(self):
        var = [3, 2, 1]
        result = sort_list(var)
        expected = [1, 2, 3]
        self.assertEqual(result, expected)

    def test_mot_list(self):
        var = {"a": "b"}
        result = sort_list(var)
        self.assertEqual(result, var)

    def test_not_same(self):
        var = [{"a": "b", "c": "d"}, {"b": "a"}]
        expected = "dictionaries do not match"
        with self.assertRaises(Exception) as exc:
            sort_list(var)
        self.assertIn(expected, str(exc.exception))

    def test_pass(self):
        var = [{"a": 2, "b": 3}, {"a": 0, "b": 1}]
        expected = [{"a": 0, "b": 1}, {"a": 2, "b": 3}]
        result = sort_list(var)
        self.assertEqual(result, expected)



# -*- coding: utf-8 -*-
# Copyright 2021 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

"""
Test plugin file for netaddr tests: in_one_network
"""

from __future__ import absolute_import, division, print_function
from ansible_collections.ansible.utils.plugins.test.in_network import (
    _in_network,
)
from ansible_collections.ansible.utils.plugins.plugin_utils.base.ipaddress_utils import (
    _validate_args,
)


__metaclass__ = type

DOCUMENTATION = """
    name: in_one_network
    author: Priyam Sahoo (@priyamsahoo)
    version_added: "2.2.0"
    short_description: Test if IP address belongs in any one of the networks in the list
    description:
        - This plugin checks if the provided IP address belongs to the provided list network addresses
    options:
        ip:
            description:
            - A string that represents an IP address
            - For example: "10.1.1.1"
            type: str
            required: True
        networks:
            description:
            - A list of string and each string represents a network address in CIDR form
            - For example: ['10.0.0.0/8', '192.168.1.0/24']
            type: list
            required: True
    notes:
"""

EXAMPLES = r"""

#### Simple examples

- name: Set network list
  ansible.builtin.set_fact:
    networks:
      - "10.0.0.0/8"
      - "192.168.1.0/24"

- name: Check if 10.1.1.1 is in the provided network list
  ansible.builtin.set_fact:
    data: "{{ '10.1.1.1' is ansible.utils.in_one_network networks }}"

# TASK [Check if 10.1.1.1 is in the provided network list] **********************
# ok: [localhost] => {
#     "ansible_facts": {
#         "data": true
#     },
#     "changed": false

- name: Set network list
  ansible.builtin.set_fact:
    networks:
      - "10.0.0.0/8"
      - "10.1.1.0/24"

- name: Check if 10.1.1.1 is not in the provided network list
  ansible.builtin.set_fact:
    data: "{{ '10.1.1.1' is not ansible.utils.in_one_network networks }}"

# TASK [Check if 10.1.1.1 is in not the provided network list] ************************
# ok: [localhost] => {
#     "ansible_facts": {
#         "data": true
#     },
#     "changed": false
# }

"""

RETURN = """
  data:
    description:
      - If jinja test satisfies plugin expression C(true)
      - If jinja test does not satisfy plugin expression C(false)
"""


def _in_one_network(ip, networks):
    """Test if an IP or network is in one network"""

    params = {"ip": ip, "networks": networks}
    _validate_args("in_one_network", DOCUMENTATION, params)

    bools = [_in_network(ip, network) for network in networks]
    if bools.count(True) == 1:
        return True
    return False


class TestModule(object):
    """ network jinja test"""

    test_map = {"in_one_network": _in_one_network}

    def tests(self):
        return self.test_map



# (c) 2020 Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

import os

import pytest

from ansible_collections.ansible.utils.tests.unit.compat import unittest
from ansible_collections.ansible.utils.plugins.sub_plugins.cli_parser.textfsm_parser import (
    CliParser,
)

textfsm = pytest.importorskip("textfsm")


class TestTextfsmParser(unittest.TestCase):
    def test_textfsm_parser(self):
        nxos_cfg_path = os.path.join(
            os.path.dirname(__file__), "fixtures", "nxos_show_version.cfg"
        )
        nxos_template_path = os.path.join(
            os.path.dirname(__file__), "fixtures", "nxos_show_version.textfsm"
        )

        with open(nxos_cfg_path) as fhand:
            nxos_show_version_output = fhand.read()

        task_args = {
            "text": nxos_show_version_output,
            "parser": {
                "name": "ansible.utils.textfsm",
                "command": "show version",
                "template_path": nxos_template_path,
            },
        }
        parser = CliParser(task_args=task_args, task_vars=[], debug=False)

        result = parser.parse()
        parsed_output = [
            {
                "BOOT_IMAGE": "bootflash:///nxos.7.0.3.I7.1.bin",
                "LAST_REBOOT_REASON": "Unknown",
                "OS": "7.0(3)I7(1)",
                "PLATFORM": "9000v",
                "UPTIME": "12 day(s), 23 hour(s), 48 minute(s), 10 second(s)",
            }
        ]
        self.assertEqual(result, {"parsed": parsed_output})

    def test_textfsm_parser_invalid_parser(self):
        fake_path = "/ /I hope this doesn't exist"
        task_args = {
            "text": "",
            "parser": {
                "name": "ansible.utils.textfsm",
                "command": "show version",
                "template_path": fake_path,
            },
        }
        parser = CliParser(task_args=task_args, task_vars=[], debug=False)
        result = parser.parse()
        errors = {
            "errors": "error while reading template_path file {0}".format(
                fake_path
            )
        }
        self.assertEqual(result, errors)



# -*- coding: utf-8 -*-
# Copyright 2020 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

DOCUMENTATION = """
    author: Ganesh Nalawade (@ganeshrn)
    name: jsonschema
    short_description: Define configurable options for jsonschema validate plugin
    description:
    - This sub plugin documentation provides the configurable options that can be passed
      to the validate plugins when C(ansible.utils.jsonschema) is used as a value for
      engine option.
    version_added: 1.0.0
    options:
      draft:
        description:
        - This option provides the jsonschema specification that should be used
          for the validating the data. The I(criteria) option in the validate
          plugin should follow the specification as mentioned by this option
        default: draft7
        choices:
        - draft3
        - draft4
        - draft6
        - draft7
        env:
        - name: ANSIBLE_VALIDATE_JSONSCHEMA_DRAFT
        vars:
        - name: ansible_validate_jsonschema_draft
    notes:
    - The value of I(data) option should be either a valid B(JSON) object or a B(JSON) string.
    - The value of I(criteria) should be B(list) of B(dict) or B(list) of B(strings) and each
      B(string) within the B(list) entry should be a valid B(dict) when read in python.
"""

import json

from ansible.module_utils._text import to_text
from ansible.module_utils.basic import missing_required_lib
from ansible.errors import AnsibleError
from ansible.module_utils.six import string_types

from ansible_collections.ansible.utils.plugins.plugin_utils.base.validate import (
    ValidateBase,
)

from ansible_collections.ansible.utils.plugins.module_utils.common.utils import (
    to_list,
)

# PY2 compatiblilty for JSONDecodeError
try:
    from json.decoder import JSONDecodeError
except ImportError:
    JSONDecodeError = ValueError

try:
    import jsonschema

    HAS_JSONSCHEMA = True
except ImportError:
    HAS_JSONSCHEMA = False


def to_path(fpath):
    return ".".join(str(index) for index in fpath)


def json_path(absolute_path):
    path = "$"
    for elem in absolute_path:
        if isinstance(elem, int):
            path += "[" + str(elem) + "]"
        else:
            path += "." + elem
    return path


class Validate(ValidateBase):
    @staticmethod
    def _check_reqs():
        """Check the prerequisites are installed for jsonschema

        :return None: In case all prerequisites are satisfised
        """
        if not HAS_JSONSCHEMA:
            raise AnsibleError(missing_required_lib("jsonschema"))

    def _check_args(self):
        """Ensure specific args are set

        :return: None: In case all arguments passed are valid
        """
        try:
            if isinstance(self._data, string_types):
                self._data = json.loads(self._data)
            else:
                self._data = json.loads(json.dumps(self._data))

        except (TypeError, JSONDecodeError) as exe:
            msg = (
                "'data' option value is invalid, value should a valid JSON."
                " Failed to read with error '{err}'".format(
                    err=to_text(exe, errors="surrogate_then_replace")
                )
            )
            raise AnsibleError(msg)

        try:
            criteria = []
            for item in to_list(self._criteria):
                if isinstance(self._criteria, string_types):
                    criteria.append(json.loads(item))
                else:
                    criteria.append(json.loads(json.dumps(item)))

            self._criteria = criteria
        except (TypeError, JSONDecodeError) as exe:
            msg = (
                "'criteria' option value is invalid, value should a valid JSON."
                " Failed to read with error '{err}'".format(
                    err=to_text(exe, errors="surrogate_then_replace")
                )
            )
            raise AnsibleError(msg)

    def validate(self):
        """Std entry point for a validate execution

        :return: Errors or parsed text as structured data
        :rtype: dict

        :example:

        The parse function of a parser should return a dict:
        {"errors": [a list of errors]}
        or
        {"parsed": obj}
        """
        self._check_reqs()
        self._check_args()

        try:
            self._validate_jsonschema()
        except Exception as exc:
            return {"errors": to_text(exc, errors="surrogate_then_replace")}

        return self._result

    def _validate_jsonschema(self):
        error_messages = None

        draft = self._get_sub_plugin_options("draft")
        error_messages = []

        for criteria in self._criteria:
            if draft == "draft3":
                validator = jsonschema.Draft3Validator(criteria)
            elif draft == "draft4":
                validator = jsonschema.Draft4Validator(criteria)
            elif draft == "draft6":
                validator = jsonschema.Draft6Validator(criteria)
            else:
                validator = jsonschema.Draft7Validator(criteria)

            validation_errors = sorted(
                validator.iter_errors(self._data), key=lambda e: e.path
            )

            if validation_errors:
                if "errors" not in self._result:
                    self._result["errors"] = []

                for validation_error in validation_errors:
                    if isinstance(
                        validation_error, jsonschema.ValidationError
                    ):
                        error = {
                            "message": validation_error.message,
                            "data_path": to_path(
                                validation_error.absolute_path
                            ),
                            "json_path": json_path(
                                validation_error.absolute_path
                            ),
                            "schema_path": to_path(
                                validation_error.relative_schema_path
                            ),
                            "relative_schema": validation_error.schema,
                            "expected": validation_error.validator_value,
                            "validator": validation_error.validator,
                            "found": validation_error.instance,
                        }
                        self._result["errors"].append(error)
                        error_message = "At '{schema_path}' {message}. ".format(
                            schema_path=error["schema_path"],
                            message=error["message"],
                        )
                        error_messages.append(error_message)
        if error_messages:
            if "msg" not in self._result:
                self._result["msg"] = "\n".join(error_messages)
            else:
                self._result["msg"] += "\n".join(error_messages)



#
# -*- coding: utf-8 -*-
# Copyright 2021 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
#


"""
The to_xml filter plugin
"""
from __future__ import absolute_import, division, print_function

__metaclass__ = type

DOCUMENTATION = """
    name: to_xml
    author: Ashwini Mhatre (@amhatre)
    version_added: "2.0.2"
    short_description: Convert given JSON string to XML
    description:
        - This plugin converts the JSON string to XML.
        - Using the parameters below- C(data|ansible.utils.to_xml)
    options:
      data:
        description:
        - The input JSON string .
        - This option represents the JSON value that is passed to the filter plugin in pipe format.
        - For example C(config_data|ansible.utils.to_xml), in this case C(config_data) represents this option.
        type: dict
        required: True
      engine:
        description:
        - Conversion library to use within the filter plugin.
        type: str
        default: xmltodict
"""

EXAMPLES = r"""

#### Simple examples with out any engine. plugin will use default value as xmltodict

- name: Define JSON data
  ansible.builtin.set_fact:
      data:
        "interface-configurations":
          "@xmlns": "http://cisco.com/ns/yang/Cisco-IOS-XR-ifmgr-cfg"
          "interface-configuration":
- debug:
    msg:  "{{ data|ansible.utils.to_xml }}"

# TASK [Define JSON data ] *************************************************************************
# task path: /Users/amhatre/ansible-collections/playbooks/test_utils_json_to_xml.yaml:5
# ok: [localhost] => {
#     "ansible_facts": {
#         "data": {
#             "interface-configurations": {
#                 "@xmlns": "http://cisco.com/ns/yang/Cisco-IOS-XR-ifmgr-cfg",
#                 "interface-configuration": null
#             }
#         }
#     },
#     "changed": false
# }
#
# TASK [debug] ***********************************************************************************************************
# task path: /Users/amhatre/ansible-collections/playbooks/test_utils_json_to_xml.yaml:13
# Loading collection ansible.utils from /Users/amhatre/ansible-collections/collections/ansible_collections/ansible/utils
# ok: [localhost] => {
#     "msg": "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n<interface-configurations xmlns=\"http://cisco.com/ns/yang/
#     Cisco-IOS-XR-ifmgr-cfg\">\n\t<interface-configuration></interface-configuration>\n</interface-configurations>"
# }

#### example2 with engine=xmltodict

- name: Define JSON data
  ansible.builtin.set_fact:
    data:
      "interface-configurations":
          "@xmlns": "http://cisco.com/ns/yang/Cisco-IOS-XR-ifmgr-cfg"
          "interface-configuration":
- debug:
    msg:  "{{ data|ansible.utils.to_xml('xmltodict') }}"

# TASK [Define JSON data ] *************************************************************************
# task path: /Users/amhatre/ansible-collections/playbooks/test_utils_json_to_xml.yaml:5
# ok: [localhost] => {
#     "ansible_facts": {
#         "data": {
#             "interface-configurations": {
#                 "@xmlns": "http://cisco.com/ns/yang/Cisco-IOS-XR-ifmgr-cfg",
#                 "interface-configuration": null
#             }
#         }
#     },
#     "changed": false
# }
# TASK [debug] ***********************************************************************************************************
# task path: /Users/amhatre/ansible-collections/playbooks/test_utils_json_to_xml.yaml:13
# Loading collection ansible.utils from /Users/amhatre/ansible-collections/collections/ansible_collections/ansible/utils
# ok: [localhost] => {
#     "msg": "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n<interface-configurations xmlns=\"http://cisco.com/ns/yang/
#     Cisco-IOS-XR-ifmgr-cfg\">\n\t<interface-configuration></interface-configuration>\n</interface-configurations>"
# }

"""

from ansible.errors import AnsibleFilterError
from jinja2.filters import environmentfilter
from ansible_collections.ansible.utils.plugins.plugin_utils.to_xml import (
    to_xml,
)
from ansible_collections.ansible.utils.plugins.module_utils.common.argspec_validate import (
    AnsibleArgSpecValidator,
)


@environmentfilter
def _to_xml(*args, **kwargs):
    """Convert the given data from json to xml."""
    keys = ["data", "engine"]
    data = dict(zip(keys, args[1:]))
    data.update(kwargs)
    aav = AnsibleArgSpecValidator(
        data=data, schema=DOCUMENTATION, name="to_xml"
    )
    valid, errors, updated_data = aav.validate()
    if not valid:
        raise AnsibleFilterError(errors)
    return to_xml(**updated_data)


class FilterModule(object):
    """ to_xml  """

    def filters(self):
        """a mapping of filter names to functions"""
        return {"to_xml": _to_xml}



#
# -*- coding: utf-8 -*-
# Copyright 2021 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
#

"""
The from_xml filter plugin
"""
from __future__ import absolute_import, division, print_function

__metaclass__ = type

DOCUMENTATION = """
    name: from_xml
    author: Ashwini Mhatre (@amhatre)
    version_added: "2.0.2"
    short_description: Convert given XML string to native python dictionary.
    description:
        - This plugin converts the XML string to a native python dictionary.
        - Using the parameters below- C(data|ansible.utils.from_xml)
    options:
      data:
        description:
        - The input XML string.
        - This option represents the XML value that is passed to the filter plugin in pipe format.
        - For example C(config_data|ansible.utils.from_xml), in this case C(config_data) represents this option.
        type: str
        required: True
      engine:
        description:
        - Conversion library to use within the filter plugin.
        type: str
        default: xmltodict
"""

EXAMPLES = r"""

#### Simple examples with out any engine. plugin will use default value as xmltodict

tasks:
  - name: convert given XML to native python dictionary
    ansible.builtin.set_fact:
      data: "
        <netconf-state xmlns=\"urn:ietf:params:xml:ns:yang:ietf-netconf-monitoring\"><schemas><schema/></schemas></netconf-state>
            "

  - debug:
      msg:  "{{ data|ansible.utils.from_xml }}"

##TASK######
# TASK [convert given XML to native python dictionary] *****************************************************************************************************
# task path: /Users/amhatre/ansible-collections/playbooks/test_utils.yaml:5
# ok: [localhost] => {
#     "ansible_facts": {
#         "data": " <netconf-state xmlns=\"urn:ietf:params:xml:ns:yang:ietf-netconf-monitoring\"><schemas><schema/></schemas></netconf-state> "
#     },
#     "changed": false
# }
#
# TASK [debug] *************************************************************************************************************************
# task path: /Users/amhatre/ansible-collections/playbooks/test_utils.yaml:13
# Loading collection ansible.utils from /Users/amhatre/ansible-collections/collections/ansible_collections/ansible/utils
# ok: [localhost] => {
#     "msg": {
#         "netconf-state": {
#             "@xmlns": "urn:ietf:params:xml:ns:yang:ietf-netconf-monitoring",
#             "schemas": {
#                 "schema": null
#             }
#         }
#     }
# }

#### example2 with engine=xmltodict

tasks:
  - name: convert given XML to native python dictionary
    ansible.builtin.set_fact:
      data: "
        <netconf-state xmlns=\"urn:ietf:params:xml:ns:yang:ietf-netconf-monitoring\"><schemas><schema/></schemas></netconf-state>
            "

  - debug:
      msg:  "{{ data|ansible.utils.from_xml('xmltodict') }}"

##TASK######
# TASK [convert given XML to native python dictionary] *****************************************************************************************************
# task path: /Users/amhatre/ansible-collections/playbooks/test_utils.yaml:5
# ok: [localhost] => {
#     "ansible_facts": {
#         "data": " <netconf-state xmlns=\"urn:ietf:params:xml:ns:yang:ietf-netconf-monitoring\"><schemas><schema/></schemas></netconf-state> "
#     },
#     "changed": false
# }
#
# TASK [debug] *************************************************************************************************************************
# task path: /Users/amhatre/ansible-collections/playbooks/test_utils.yaml:13
# Loading collection ansible.utils from /Users/amhatre/ansible-collections/collections/ansible_collections/ansible/utils
# ok: [localhost] => {
#     "msg": {
#         "netconf-state": {
#             "@xmlns": "urn:ietf:params:xml:ns:yang:ietf-netconf-monitoring",
#             "schemas": {
#                 "schema": null
#             }
#         }
#     }
# }
"""

from ansible.errors import AnsibleFilterError
from jinja2.filters import environmentfilter
from ansible_collections.ansible.utils.plugins.plugin_utils.from_xml import (
    from_xml,
)
from ansible_collections.ansible.utils.plugins.module_utils.common.argspec_validate import (
    AnsibleArgSpecValidator,
)


@environmentfilter
def _from_xml(*args, **kwargs):
    """Convert the given data from xml to json."""

    keys = ["data", "engine"]
    data = dict(zip(keys, args[1:]))
    data.update(kwargs)
    aav = AnsibleArgSpecValidator(
        data=data, schema=DOCUMENTATION, name="from_xml"
    )
    valid, errors, updated_data = aav.validate()
    if not valid:
        raise AnsibleFilterError(errors)
    return from_xml(**updated_data)


class FilterModule(object):
    """ from_xml  """

    def filters(self):

        """a mapping of filter names to functions"""
        return {"from_xml": _from_xml}
